#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

import warnings
from unittest import TestCase

from holidays.calendars.germany_school import GERMANY_SCHOOL_HOLIDAYS
from holidays.constants import CATHOLIC, PUBLIC, SCHOOL
from holidays.countries.germany import Germany
from tests.common import CommonCountryTests


class TestGermany(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Germany, with_subdiv_categories=True)

    def setUp(self):
        super().setUp()
        warnings.simplefilter("ignore", category=DeprecationWarning)

    def test_subdiv_deprecation(self):
        self.assertDeprecatedSubdivisions("This subdivision is deprecated and will be removed")

    def test_deprecated(self):
        self.assertEqual(
            sorted(Germany(subdiv="BYP", years=2023).keys()),
            sorted(Germany(subdiv="BY", years=2023).keys()),
        )

    def test_no_holidays(self):
        super().test_no_holidays()

        for subdiv in Germany.subdivisions:
            self.assertNoHolidays(Germany(years=self.start_year - 1, subdiv=subdiv))
            self.assertNoHolidays(
                Germany(subdiv=subdiv, years=self.start_year - 1, categories=SCHOOL)
            )
            if subdiv in {"BY", "SN", "TH"}:
                self.assertNoHolidays(
                    Germany(subdiv=subdiv, years=self.start_year - 1, categories=CATHOLIC)
                )
        self.assertNoHolidays(Germany(years=2025, categories=SCHOOL))

    def test_special_holidays(self):
        # 2017's Reformation Day is tested in test_reformation_day.
        be_dts = (
            "2020-05-08",
            "2025-05-08",
            "2028-06-17",
        )
        for subdiv, holidays in self.subdiv_holidays.items():
            # Berlin.
            if subdiv == "BE":
                self.assertHoliday(holidays, be_dts)
            else:
                self.assertNoHoliday(holidays, be_dts)

    def test_new_years_day(self):
        self.assertHolidayName("Neujahr", (f"{year}-01-01" for year in self.full_range))

    def test_epiphany(self):
        name = "Heilige Drei Könige"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            # Bayern, Sachsen, Thüringen, Augsburg.
            if subdiv in {"BW", "BY", "ST", "Augsburg"}:
                self.assertHolidayName(
                    name, holidays, (f"{year}-01-06" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_womens_day(self):
        name = "Frauentag"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            match subdiv:
                # Berlin - 2019.
                case "BE":
                    self.assertHolidayName(
                        name, holidays, (f"{year}-03-08" for year in range(2019, self.end_year))
                    )
                    self.assertNoHolidayName(name, holidays, range(self.start_year, 2019))
                # Mecklenburg-Vorpommern - 2023.
                case "MV":
                    self.assertHolidayName(
                        name, holidays, (f"{year}-03-08" for year in range(2023, self.end_year))
                    )
                    self.assertNoHolidayName(name, holidays, range(self.start_year, 2023))
                case _:
                    self.assertNoHolidayName(name, holidays)

    def test_good_friday(self):
        name = "Karfreitag"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_sunday(self):
        name = "Ostersonntag"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            # Brandenburg.
            if subdiv == "BB":
                self.assertHolidayName(
                    name,
                    holidays,
                    "2020-04-12",
                    "2021-04-04",
                    "2022-04-17",
                    "2023-04-09",
                    "2024-03-31",
                    "2025-04-20",
                )
                self.assertHolidayName(name, holidays, self.full_range)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_easter_monday(self):
        name = "Ostermontag"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_ascension_day(self):
        name = "Christi Himmelfahrt"
        self.assertHolidayName(
            name,
            "2020-05-21",
            "2021-05-13",
            "2022-05-26",
            "2023-05-18",
            "2024-05-09",
            "2025-05-29",
        )
        self.assertHolidayName(name, self.full_range)

    def test_labor_day(self):
        self.assertHolidayName("Erster Mai", (f"{year}-05-01" for year in self.full_range))

    def test_whit_sunday(self):
        name = "Pfingstsonntag"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            # Brandenburg.
            if subdiv == "BB":
                self.assertHolidayName(
                    name,
                    holidays,
                    "2020-05-31",
                    "2021-05-23",
                    "2022-06-05",
                    "2023-05-28",
                    "2024-05-19",
                    "2025-06-08",
                )
                self.assertHolidayName(name, holidays, self.full_range)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_whit_monday(self):
        name = "Pfingstmontag"
        self.assertHolidayName(
            name,
            "2020-06-01",
            "2021-05-24",
            "2022-06-06",
            "2023-05-29",
            "2024-05-20",
            "2025-06-09",
        )
        self.assertHolidayName(name, self.full_range)

    def test_corpus_christi(self):
        name = "Fronleichnam"
        self.assertNoHolidayName(name)
        dts = (
            "2020-06-11",
            "2021-06-03",
            "2022-06-16",
            "2023-06-08",
            "2024-05-30",
            "2025-06-19",
        )
        for subdiv, holidays in self.subdiv_holidays.items():
            # Baden-Württemberg, Bayern, Hessen, Nordrhein-Westfalen,
            # Rheinland-Pfalz, Saarland, Augsburg.
            if subdiv in {"BW", "BY", "HE", "NW", "RP", "SL", "Augsburg"}:
                self.assertHolidayName(name, holidays, dts)
                self.assertHolidayName(name, holidays, self.full_range)
            else:
                self.assertNoHolidayName(name, holidays)

        # Sachsen, Thüringen.
        self.assertSubdivSnCatholicHolidayName(name, dts)
        self.assertSubdivSnCatholicHolidayName(name, self.full_range)
        self.assertSubdivThCatholicHolidayName(name, dts)
        self.assertSubdivThCatholicHolidayName(name, self.full_range)

    def test_augsburg_peace_festival(self):
        name = "Augsburger Hohes Friedensfest"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            # Augsburg.
            if subdiv == "Augsburg":
                self.assertHolidayName(
                    name, holidays, (f"{year}-08-08" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_assumption_day(self):
        name = "Mariä Himmelfahrt"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            # Saarland.
            if subdiv in {"SL", "Augsburg"}:
                self.assertHolidayName(
                    name, holidays, (f"{year}-08-15" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

        # Bayern.
        self.assertSubdivByCatholicHolidayName(name, (f"{year}-08-15" for year in self.full_range))

    def test_world_childrens_day(self):
        name = "Weltkindertag"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            # Thüringen.
            if subdiv == "TH":
                self.assertHolidayName(
                    name, holidays, (f"{year}-09-20" for year in range(2019, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2019))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_german_unity_day(self):
        self.assertHolidayName(
            "Tag der Deutschen Einheit", (f"{year}-10-03" for year in self.full_range)
        )

    def test_reformation_day(self):
        name = "Reformationstag"
        for subdiv, holidays in self.subdiv_holidays.items():
            match subdiv:
                # Brandenburg, Mecklenburg-Vorpommern, Sachsen, Sachsen-Anhalt, Thüringen.
                case "BB" | "MV" | "SN" | "ST" | "TH":
                    self.assertHolidayName(
                        name, holidays, (f"{year}-10-31" for year in self.full_range)
                    )
                # Bremen, Hamburg, Niedersachsen, Schleswig-Holstein.
                case "HB" | "HH" | "NI" | "SH":
                    # While these subdivisions started their holiday observance in 2018,
                    # this is de facto implemented in 2017's nationwide special observance.
                    self.assertHolidayName(
                        name, holidays, (f"{year}-10-31" for year in range(2017, self.end_year))
                    )
                    self.assertNoHolidayName(name, holidays, range(self.start_year, 2017))
                case _:
                    self.assertHolidayName(name, holidays, "2017-10-31")
                    self.assertNoHolidayName(
                        name, holidays, range(self.start_year, 2017), range(2018, self.end_year)
                    )
        self.assertHolidayName(name, "2017-10-31")

    def test_all_saints_day(self):
        name = "Allerheiligen"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            # Baden-Württemberg, Bayern, Nordrhein-Westfalen, Rheinland-Pfalz, Saarland, Augsburg.
            if subdiv in {"BW", "BY", "NW", "RP", "SL", "Augsburg"}:
                self.assertHolidayName(
                    name, holidays, (f"{year}-11-01" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_repentance_and_prayer_day(self):
        name = "Buß- und Bettag"
        dts_pre_1995 = (
            "1991-11-20",
            "1992-11-18",
            "1993-11-17",
            "1994-11-16",
        )
        dts = (
            "2020-11-18",
            "2021-11-17",
            "2022-11-16",
            "2023-11-22",
            "2024-11-20",
            "2025-11-19",
        )
        for subdiv, holidays in self.subdiv_holidays.items():
            # Sachsen.
            if subdiv == "SN":
                self.assertHolidayName(name, holidays, dts, dts_pre_1995)
                self.assertHolidayName(name, holidays, self.full_range)
            else:
                self.assertHolidayName(name, holidays, dts_pre_1995)
                self.assertNoHolidayName(name, holidays, range(1995, self.end_year))
        self.assertHolidayName(name, dts_pre_1995)
        self.assertNoHolidayName(name, range(1995, self.end_year))

    def test_christmas_day(self):
        self.assertHolidayName(
            "Erster Weihnachtstag", (f"{year}-12-25" for year in self.full_range)
        )

    def test_second_day_of_christmas(self):
        self.assertHolidayName(
            "Zweiter Weihnachtstag", (f"{year}-12-26" for year in self.full_range)
        )

    def test_all_states_present_for_all_dataset_years(self):
        expected_subdivs = set(Germany.subdivisions) - {"Augsburg"}
        for year, year_data in GERMANY_SCHOOL_HOLIDAYS.items():
            self.assertEqual(set(year_data), expected_subdivs, year)

    def test_all_school_holiday_ids_are_mapped(self):
        known_ids = set(Germany._get_school_holiday_names())
        dataset_ids = {
            holiday_id
            for year_data in GERMANY_SCHOOL_HOLIDAYS.values()
            for holidays in year_data.values()
            for *_, holiday_id in holidays
        }
        self.assertEqual(dataset_ids, known_ids)

    def test_school_holidays(self):
        self.assertSubdivBeSchoolHolidayName(
            "Himmelfahrts-/Pfingstferien",
            "2004-05-21",
        )
        self.assertSubdivBeSchoolHolidayName(
            "Sommerferien",
            "2012-06-20",
            "2013-06-19",
            "2014-07-09",
            "2016-07-20",
        )
        self.assertSubdivBeSchoolHolidayName(
            "Herbstferien",
            "2025-10-20",
            "2025-11-01",
        )
        self.assertSubdivBwSchoolHolidayName(
            "Winterferien",
            "1991-02-11",
            "1991-02-16",
        )
        self.assertSubdivBwSchoolHolidayName(
            "Sommerferien",
            "1991-07-11",
            "1991-08-24",
        )
        self.assertSubdivBwSchoolHolidayName("Herbstferien", "2022-11-04")
        self.assertSubdivBwSchoolHolidayName(
            "Himmelfahrts-/Pfingstferien",
            "2025-06-10",
            "2025-06-20",
        )
        self.assertSubdivBySchoolHolidayName(
            "Sommerferien",
            "2025-08-01",
            "2025-08-31",
        )
        self.assertSubdivBySchoolHolidayName(
            "Weihnachtsferien",
            "2026-01-02",
            "2026-12-24",
        )
        self.assertSubdivHbSchoolHolidayName(
            "Winterferien",
            "2009-02-02",
            "2009-02-03",
        )
        self.assertSubdivHhSchoolHolidayName(
            "Weihnachtsferien",
            "2004-01-30",
            "2004-12-22",
        )
        self.assertSubdivNiSchoolHolidayName(
            "Winterferien",
            "2001-01-29",
            "2001-01-30",
        )
        self.assertSubdivNwSchoolHolidayName(
            "Sommerferien",
            "2026-07-20",
            "2026-09-01",
        )
        self.assertSubdivShSchoolHolidayName(
            "Sommerferien",
            "2026-07-04",
            "2026-08-15",
        )
        self.assertSubdivSlSchoolHolidayName(
            "Himmelfahrts-/Pfingstferien",
            "2004-05-21",
            "2004-06-11",
        )
        self.assertSubdivSnSchoolHolidayName(
            "Winterferien",
            "2026-02-09",
            "2026-02-21",
        )
        self.assertSubdivStSchoolHolidayName(
            "Herbstferien",
            "2004-10-18",
            "2004-10-23",
        )
        self.assertSubdivStSchoolHolidayName(
            "Winterferien",
            "2006-02-01",
            "2006-02-10",
        )
        self.assertSubdivThSchoolHolidayName(
            "Sommerferien",
            "2001-06-28",
            "2001-08-08",
        )

    def test_school_holidays_subdivision_aliases(self):
        self.assertHolidayName(
            "Sommerferien",
            Germany(subdiv="Bayern", years=2025, categories=SCHOOL),
            "2025-08-01",
            "2025-08-31",
        )

        self.assertHolidayName(
            "Sommerferien",
            Germany(subdiv="Augsburg", years=2025, categories=SCHOOL),
            "2025-08-01",
            "2025-08-31",
        )

        self.assertHolidayName(
            "Sommerferien",
            Germany(subdiv="BYP", years=2025, categories=SCHOOL),
            "2025-08-01",
            "2025-08-31",
        )

    def test_school_and_public_categories(self):
        self.assertHolidayName(
            "Frauentag",
            Germany(subdiv="BE", years=2025, categories=(PUBLIC, SCHOOL)),
            "2025-03-08",
        )
        self.assertHolidayName(
            "Herbstferien",
            Germany(subdiv="BE", years=2025, categories=(PUBLIC, SCHOOL)),
            "2025-10-20",
        )
        self.assertHolidayName(
            "Neujahr",
            Germany(subdiv="Bayern", years=2025, categories=(PUBLIC, SCHOOL)),
            "2025-01-01",
        )
        self.assertHolidayName(
            "Sommerferien",
            Germany(subdiv="Bayern", years=2025, categories=(PUBLIC, SCHOOL)),
            "2025-08-01",
        )

    def test_public_category_excludes_school_holidays(self):
        self.assertHolidayName(
            "Neujahr",
            Germany(subdiv="BY", years=2025, categories=PUBLIC),
            "2025-01-01",
        )
        self.assertNoHolidayName(
            "Sommerferien",
            Germany(subdiv="BY", years=2025, categories=PUBLIC),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Neujahr"),
            ("2022-01-06", "Heilige Drei Könige"),
            ("2022-03-08", "Frauentag"),
            ("2022-04-15", "Karfreitag"),
            ("2022-04-17", "Ostersonntag"),
            ("2022-04-18", "Ostermontag"),
            ("2022-05-01", "Erster Mai"),
            ("2022-05-26", "Christi Himmelfahrt"),
            ("2022-06-05", "Pfingstsonntag"),
            ("2022-06-06", "Pfingstmontag"),
            ("2022-06-16", "Fronleichnam"),
            ("2022-08-08", "Augsburger Hohes Friedensfest"),
            ("2022-08-15", "Mariä Himmelfahrt"),
            ("2022-09-20", "Weltkindertag"),
            ("2022-10-03", "Tag der Deutschen Einheit"),
            ("2022-10-31", "Reformationstag"),
            ("2022-11-01", "Allerheiligen"),
            ("2022-11-16", "Buß- und Bettag"),
            ("2022-12-25", "Erster Weihnachtstag"),
            ("2022-12-26", "Zweiter Weihnachtstag"),
            categories=(CATHOLIC, PUBLIC),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-01-06", "Epiphany"),
            ("2022-03-08", "Women's Day"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-17", "Easter Sunday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-05-01", "Labor Day"),
            ("2022-05-26", "Ascension Day"),
            ("2022-06-05", "Whit Sunday"),
            ("2022-06-06", "Whit Monday"),
            ("2022-06-16", "Corpus Christi"),
            ("2022-08-08", "Augsburg Peace Festival"),
            ("2022-08-15", "Assumption Day"),
            ("2022-09-20", "World Children's Day"),
            ("2022-10-03", "German Unity Day"),
            ("2022-10-31", "Reformation Day"),
            ("2022-11-01", "All Saints' Day"),
            ("2022-11-16", "Repentance and Prayer Day"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Second Day of Christmas"),
            categories=(CATHOLIC, PUBLIC),
        )

    def test_l10n_th(self):
        self.assertLocalizedHolidays(
            "th",
            ("2022-01-01", "วันขึ้นปีใหม่"),
            ("2022-01-06", "วันสมโภชพระคริสต์แสดงองค์"),
            ("2022-03-08", "วันสตรี"),
            ("2022-04-15", "วันศุกร์ประเสริฐ"),
            ("2022-04-17", "วันอาทิตย์อีสเตอร์"),
            ("2022-04-18", "วันจันทร์อีสเตอร์"),
            ("2022-05-01", "วันแรงงาน"),
            ("2022-05-26", "วันสมโภชพระเยซูเจ้าเสด็จขึ้นสวรรค์"),
            ("2022-06-05", "วันสมโภชพระจิตเจ้า"),
            ("2022-06-06", "วันจันทร์หลังวันสมโภชพระจิตเจ้า"),
            ("2022-06-16", "วันสมโภชพระคริสตวรกาย"),
            ("2022-08-08", "วันเทศกาลสันติภาพเอาก์สบวร์ก"),
            ("2022-08-15", "วันสมโภชแม่พระรับเกียรติยกขึ้นสวรรค์"),
            ("2022-09-20", "วันเด็กสากล"),
            ("2022-10-03", "วันรวมชาติเยอรมัน"),
            ("2022-10-31", "วันแห่งการปฏิรูป"),
            ("2022-11-01", "วันสมโภชนักบุญทั้งหลาย"),
            ("2022-11-16", "วันแห่งการอธิษฐานและการกลับใจ"),
            ("2022-12-25", "วันคริสต์มาสวันแรก"),
            ("2022-12-26", "วันคริสต์มาสวันที่สอง"),
            categories=(CATHOLIC, PUBLIC),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-01-06", "Богоявлення"),
            ("2022-03-08", "Жіночий день"),
            ("2022-04-15", "Страсна пʼятниця"),
            ("2022-04-17", "Великдень"),
            ("2022-04-18", "Великодній понеділок"),
            ("2022-05-01", "День праці"),
            ("2022-05-26", "Вознесіння Господнє"),
            ("2022-06-05", "Трійця"),
            ("2022-06-06", "День Святого Духа"),
            ("2022-06-16", "Свято Тіла і Крові Христових"),
            ("2022-08-08", "Аугсбурзьке свято миру"),
            ("2022-08-15", "Внебовзяття Пресвятої Діви Марії"),
            ("2022-09-20", "Всесвітній день дітей"),
            ("2022-10-03", "День німецької єдності"),
            ("2022-10-31", "День Реформації"),
            ("2022-11-01", "День усіх святих"),
            ("2022-11-16", "День молитви та покаяння"),
            ("2022-12-25", "Перший день Різдва"),
            ("2022-12-26", "Другий день Різдва"),
            categories=(CATHOLIC, PUBLIC),
        )

    def test_l10n_default_school(self):
        default_holidays = Germany(subdiv="BB", years=2025, categories=SCHOOL)
        for dt, name in (
            ("2025-02-03", "Winterferien"),
            ("2025-04-14", "Oster-/Frühjahrsferien"),
            ("2025-06-10", "Himmelfahrts-/Pfingstferien"),
            ("2025-07-24", "Sommerferien"),
            ("2025-10-20", "Herbstferien"),
            ("2025-12-22", "Weihnachtsferien"),
        ):
            self.assertHolidayName(name, default_holidays, dt)

    def test_l10n_en_us_school(self):
        en_us_holidays = Germany(subdiv="BB", years=2025, language="en_US", categories=SCHOOL)
        for dt, name in (
            ("2025-02-03", "Winter Break"),
            ("2025-04-14", "Easter/Spring Break"),
            ("2025-06-10", "Ascension/Whit Break"),
            ("2025-07-24", "Summer Break"),
            ("2025-10-20", "Autumn Break"),
            ("2025-12-22", "Christmas Break"),
        ):
            self.assertHolidayName(name, en_us_holidays, dt)

    def test_l10n_th_school(self):
        th_holidays = Germany(subdiv="BB", years=2025, language="th", categories=SCHOOL)
        for dt, name in (
            ("2025-02-03", "ปิดเทอมฤดูหนาว"),
            ("2025-04-14", "ปิดเทอมอีสเตอร์/ฤดูใบไม้ผลิ"),
            ("2025-06-10", "ปิดเทอมวันสมโภชพระเยซูเจ้าเสด็จสู่สวรรค์/เพ็นเทคอสต์"),
            ("2025-07-24", "ปิดเทอมฤดูร้อน"),
            ("2025-10-20", "ปิดเทอมฤดูใบไม้ร่วง"),
            ("2025-12-22", "ปิดเทอมคริสต์มาส"),
        ):
            self.assertHolidayName(name, th_holidays, dt)

    def test_l10n_uk_school(self):
        uk_holidays = Germany(subdiv="BB", years=2025, language="uk", categories=SCHOOL)
        for dt, name in (
            ("2025-02-03", "Зимові канікули"),
            ("2025-04-14", "Великодні/весняні канікули"),
            ("2025-06-10", "Канікули на Вознесіння/Трійцю"),
            ("2025-07-24", "Літні канікули"),
            ("2025-10-20", "Осінні канікули"),
            ("2025-12-22", "Різдвяні канікули"),
        ):
            self.assertHolidayName(name, uk_holidays, dt)



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.comoros import Comoros
from tests.common import CommonCountryTests


class TestComoros(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Comoros)

    def test_special_holidays(self):
        self.assertHolidayName("National Holiday", "2024-04-13")
        self.assertHolidayName("Election Partial Day Holiday", "2025-01-30")

    def test_new_years_day(self):
        self.assertHolidayName("New Year's Day", (f"{year}-01-01" for year in self.full_range))

    def test_cheikh_al_maarouf_day(self):
        self.assertHolidayName(
            "Cheikh al Maarouf Day", (f"{year}-03-18" for year in self.full_range)
        )

    def test_labor_day(self):
        self.assertHolidayName("Labour Day", (f"{year}-05-01" for year in self.full_range))

    def test_national_day(self):
        self.assertHolidayName("National Day", (f"{year}-07-06" for year in self.full_range))

    def test_maore_day(self):
        name = "Maore Day"
        self.assertNoHolidayName(name, range(self.start_year, 2006))
        self.assertHolidayName(name, (f"{year}-11-12" for year in range(2006, self.end_year)))

    def test_christmas_day(self):
        self.assertHolidayName("Christmas Day", (f"{year}-12-25" for year in self.full_range))

    def test_islamic_new_year(self):
        name = "Islamic New Year"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-08-20",
            "2021-08-09",
            "2022-07-30",
            "2023-07-19",
            "2024-07-07",
            "2025-06-26",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_prophets_birthday(self):
        name = "Prophet's Birthday"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-10-29",
            "2021-10-19",
            "2022-10-08",
            "2023-09-27",
            "2024-09-15",
            "2025-09-04",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_isra_and_miraj(self):
        name = "Isra' and Mi'raj"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-03-22",
            "2021-03-11",
            "2022-02-28",
            "2023-02-18",
            "2024-02-08",
            "2025-01-27",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_eid_al_fitr(self):
        name = "Eid al-Fitr"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-05-24",
            "2021-05-13",
            "2022-05-02",
            "2023-04-21",
            "2024-04-10",
            "2025-03-30",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_eid_al_adha(self):
        name = "Eid al-Adha"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-07-31",
            "2021-07-20",
            "2022-07-09",
            "2023-06-28",
            "2024-06-16",
            "2025-06-06",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_2022(self):
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "New Year's Day"),
            ("2022-02-28", "Isra' and Mi'raj (estimated)"),
            ("2022-03-18", "Cheikh al Maarouf Day"),
            ("2022-05-01", "Labour Day"),
            ("2022-05-02", "Eid al-Fitr (estimated)"),
            ("2022-05-03", "Eid al-Fitr (estimated)"),
            ("2022-05-04", "Eid al-Fitr (estimated)"),
            ("2022-07-06", "National Day"),
            ("2022-07-09", "Eid al-Adha (estimated)"),
            ("2022-07-10", "Eid al-Adha (estimated)"),
            ("2022-07-30", "Islamic New Year (estimated)"),
            ("2022-10-08", "Prophet's Birthday (estimated)"),
            ("2022-11-12", "Maore Day"),
            ("2022-12-25", "Christmas Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.turkmenistan import Turkmenistan
from tests.common import CommonCountryTests


class TestTurkmenistan(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        years = range(1992, 2050)
        super().setUpClass(Turkmenistan, years=years)
        cls.no_estimated_holidays = Turkmenistan(years=years, islamic_show_estimated=False)

    def test_new_years_day(self):
        name = "Täze ýyl"
        self.assertHolidayName(name, (f"{year}-01-01" for year in range(1992, 2050)))
        obs_dt = (
            "2012-01-02",
            "2017-01-02",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (dynç güni)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_state_flag_day(self):
        name = "Türkmenistanyň Döwlet baýdagynyň güni"
        self.assertHolidayName(name, (f"{year}-02-19" for year in range(1995, 2018)))
        self.assertNoHolidayName(name, range(1992, 1995), range(2018, 2050))
        obs_dt = (
            "2012-02-20",
            "2017-02-20",
        )
        self.assertHolidayName(f"{name} (dynç güni)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_womens_day(self):
        name = "Halkara zenanlar güni"
        self.assertHolidayName(
            name, (f"{year}-03-08" for year in (*range(1992, 2001), *range(2008, 2050)))
        )
        self.assertNoHolidayName(name, range(2001, 2008))
        obs_dt = (
            "2015-03-09",
            "2020-03-09",
        )
        self.assertHolidayName(f"{name} (dynç güni)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_spring_festival(self):
        name = "Milli bahar baýramy"
        self.assertHolidayName(name, (f"{year}-03-20" for year in range(2001, 2008)))
        self.assertHolidayName(name, (f"{year}-03-21" for year in range(1992, 2050)))
        self.assertHolidayName(name, (f"{year}-03-22" for year in range(1992, 2050)))
        obs_dt = (
            "2010-03-23",
            "2015-03-23",
            "2020-03-23",
            "2021-03-23",
        )
        self.assertHolidayName(f"{name} (dynç güni)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_victory_day(self):
        name = "1941-1945-nji ýyllaryň Beýik Watançylyk urşunda ýeňiş güni"
        self.assertHolidayName(name, (f"{year}-05-09" for year in range(1992, 2018)))
        self.assertNoHolidayName(name, range(2018, 2050))
        obs_dt = ("2010-05-10",)
        self.assertHolidayName(f"{name} (dynç güni)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_unity_and_revival_day(self):
        name = "Galkynyş, Agzybirlik we Magtymguly Pyragynyň şygryýet güni"
        self.assertHolidayName(name, (f"{year}-05-18" for year in range(2009, 2015)))
        self.assertNoHolidayName(name, range(1992, 2009), range(2015, 2050))
        obs_dt = ("2014-05-19",)
        self.assertHolidayName(f"{name} (dynç güni)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_state_flag_and_constitution_day_pre2018(self):
        name = "Türkmenistanyň Konstitusiýasynyň we Makhtumkuli Pyragynyň şygryýet güni"
        self.assertHolidayName(name, (f"{year}-05-18" for year in range(2015, 2018)))
        self.assertNoHolidayName(name, range(1992, 2015), range(2018, 2050))

    def test_state_flag_and_constitution_day_post2018(self):
        name = "Türkmenistanyň Konstitusiýasynyň we Döwlet baýdagynyň güni"
        self.assertHolidayName(name, (f"{year}-05-18" for year in range(2018, 2050)))
        self.assertNoHolidayName(name, range(1992, 2018))
        obs_dt = ("2025-05-19",)
        self.assertHolidayName(f"{name} (dynç güni)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_independence_day(self):
        name = "Türkmenistanyň Garaşsyzlyk güni"
        self.assertHolidayName(name, (f"{year}-10-27" for year in range(1992, 2018)))
        self.assertHolidayName(name, (f"{year}-10-28" for year in range(2008, 2018)))
        self.assertHolidayName(name, (f"{year}-09-27" for year in range(2018, 2050)))
        obs_dt = (
            "2012-10-29",
            "2013-10-29",
            "2020-09-28",
        )
        self.assertHolidayName(f"{name} (dynç güni)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_memorial_day(self):
        name = "Hatyra güni"
        self.assertHolidayName(
            name, (f"{year}-10-06" for year in (*range(1995, 2009), *range(2015, 2050)))
        )
        self.assertHolidayName(name, (f"{year}-01-12" for year in range(2009, 2015)))
        self.assertNoHolidayName(name, range(1992, 1995))
        obs_dt = (
            "2014-01-13",
            "2019-10-07",
            "2024-10-07",
        )
        self.assertHolidayName(f"{name} (dynç güni)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_national_memorial_day(self):
        name = "Milli ýatlama güni"
        self.assertHolidayName(name, (f"{year}-10-06" for year in range(2009, 2015)))
        self.assertNoHolidayName(name, range(1992, 2009), range(2015, 2050))
        obs_dt = ("2013-10-07",)
        self.assertHolidayName(f"{name} (dynç güni)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_neutrality_day(self):
        name = "Halkara Bitaraplyk güni"
        self.assertHolidayName(name, (f"{year}-12-12" for year in range(1995, 2050)))
        self.assertNoHolidayName(name, range(1992, 1995))
        obs_dt = (
            "2010-12-13",
            "2021-12-13",
        )
        self.assertHolidayName(f"{name} (dynç güni)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_eid_al_fitr(self):
        name = "Oraza baýramy"
        dts = (
            "2021-05-13",
            "2022-05-02",
            "2023-04-21",
            "2024-04-10",
            "2025-03-30",
        )
        self.assertHolidayName(name, dts)
        self.assertHolidayName(name, self.no_estimated_holidays, range(1992, 2050))
        obs_dt = (
            "2012-08-20",
            "2020-05-25",
            "2025-03-31",
        )
        self.assertHolidayName(f"{name} (dynç güni)", self.no_estimated_holidays, obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_eid_al_adha(self):
        name = "Gurban baýramy"
        dts = (
            "2021-07-20",
            "2022-07-09",
            "2023-06-28",
            "2024-06-16",
            "2025-06-06",
        )
        self.assertHolidayName(name, dts)
        self.assertHolidayName(name, self.no_estimated_holidays, range(1992, 2050))
        obs_dt = (
            "2014-10-07",
            "2019-08-12",
            "2024-06-17",
        )
        self.assertHolidayName(f"{name} (dynç güni)", self.no_estimated_holidays, obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2024-01-01", "Täze ýyl"),
            ("2024-03-08", "Halkara zenanlar güni"),
            ("2024-03-21", "Milli bahar baýramy"),
            ("2024-03-22", "Milli bahar baýramy"),
            ("2024-04-10", "Oraza baýramy"),
            ("2024-05-18", "Türkmenistanyň Konstitusiýasynyň we Döwlet baýdagynyň güni"),
            ("2024-06-16", "Gurban baýramy"),
            ("2024-06-17", "Gurban baýramy (dynç güni)"),
            ("2024-09-27", "Türkmenistanyň Garaşsyzlyk güni"),
            ("2024-10-06", "Hatyra güni"),
            ("2024-10-07", "Hatyra güni (dynç güni)"),
            ("2024-12-12", "Halkara Bitaraplyk güni"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2024-01-01", "New Year's Day"),
            ("2024-03-08", "International Women's Day"),
            ("2024-03-21", "Spring Festival"),
            ("2024-03-22", "Spring Festival"),
            ("2024-04-10", "Eid al-Fitr"),
            ("2024-05-18", "Constitution and State Flag Day"),
            ("2024-06-16", "Eid al-Adha"),
            ("2024-06-17", "Eid al-Adha (observed)"),
            ("2024-09-27", "Independence Day"),
            ("2024-10-06", "Memorial Day"),
            ("2024-10-07", "Memorial Day (observed)"),
            ("2024-12-12", "International Neutrality Day"),
        )

    def test_l10n_ru(self):
        self.assertLocalizedHolidays(
            "ru",
            ("2024-01-01", "Новый год"),
            ("2024-03-08", "Международный женский день"),
            ("2024-03-21", "Национальный праздник весны"),
            ("2024-03-22", "Национальный праздник весны"),
            ("2024-04-10", "Ораза байрам"),
            (
                "2024-05-18",
                "День Конституции Туркменистана и Государственного флага Туркменистана",
            ),
            ("2024-06-16", "Курбан байрам"),
            ("2024-06-17", "Курбан байрам (выходной)"),
            ("2024-09-27", "День независимости"),
            ("2024-10-06", "День поминовения"),
            ("2024-10-07", "День поминовения (выходной)"),
            ("2024-12-12", "Международный день нейтралитета"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.lithuania import Lithuania
from tests.common import CommonCountryTests


class TestLithuania(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Lithuania)

    def test_new_years_day(self):
        self.assertHolidayName("Naujųjų metų diena", (f"{year}-01-01" for year in self.full_range))

    def test_day_of_restoration_of_the_state_of_lithuania(self):
        self.assertHolidayName(
            "Lietuvos valstybės atkūrimo diena", (f"{year}-02-16" for year in self.full_range)
        )

    def test_day_of_restoration_of_independence_of_lithuania(self):
        self.assertHolidayName(
            "Lietuvos nepriklausomybės atkūrimo diena",
            (f"{year}-03-11" for year in self.full_range),
        )

    def test_easter_sunday(self):
        name = "Šv. Velykos"
        self.assertHolidayName(
            name,
            "2020-04-12",
            "2021-04-04",
            "2022-04-17",
            "2023-04-09",
            "2024-03-31",
            "2025-04-20",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Antroji šv. Velykų diena"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_labor_day(self):
        self.assertHolidayName(
            "Tarptautinė darbo diena", (f"{year}-05-01" for year in self.full_range)
        )

    def test_mothers_day(self):
        name = "Motinos diena"
        self.assertHolidayName(
            name,
            "2020-05-03",
            "2021-05-02",
            "2022-05-01",
            "2023-05-07",
            "2024-05-05",
            "2025-05-04",
        )
        self.assertHolidayName(name, self.full_range)

    def test_fathers_day(self):
        name = "Tėvo diena"
        self.assertHolidayName(
            name,
            "2020-06-07",
            "2021-06-06",
            "2022-06-05",
            "2023-06-04",
            "2024-06-02",
            "2025-06-01",
        )
        self.assertHolidayName(name, self.full_range)

    def test_day_of_dew_and_saint_john(self):
        name = "Rasos ir Joninių diena"
        self.assertHolidayName(name, (f"{year}-06-24" for year in range(2003, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2003))

    def test_statehood_day(self):
        name = "Valstybės (Lietuvos karaliaus Mindaugo karūnavimo) ir Tautiškos giesmės diena"
        self.assertHolidayName(name, (f"{year}-07-06" for year in range(1991, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1991))

    def test_assumption_day(self):
        self.assertHolidayName(
            "Žolinė (Švč. Mergelės Marijos ėmimo į dangų diena)",
            (f"{year}-08-15" for year in self.full_range),
        )

    def test_all_saints_day(self):
        self.assertHolidayName(
            "Visų Šventųjų diena", (f"{year}-11-01" for year in self.full_range)
        )

    def test_all_souls_day(self):
        name = "Mirusiųjų atminimo (Vėlinių) diena"
        self.assertHolidayName(name, (f"{year}-11-02" for year in range(2020, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2020))

    def test_christmas_eve(self):
        self.assertHolidayName("Kūčių diena", (f"{year}-12-24" for year in self.full_range))

    def test_christmas_day(self):
        self.assertHolidayName(
            "Šv. Kalėdų pirma diena", (f"{year}-12-25" for year in self.full_range)
        )

    def test_second_day_of_christmas(self):
        self.assertHolidayName(
            "Šv. Kalėdų antra diena", (f"{year}-12-26" for year in self.full_range)
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Naujųjų metų diena"),
            ("2022-02-16", "Lietuvos valstybės atkūrimo diena"),
            ("2022-03-11", "Lietuvos nepriklausomybės atkūrimo diena"),
            ("2022-04-17", "Šv. Velykos"),
            ("2022-04-18", "Antroji šv. Velykų diena"),
            ("2022-05-01", "Motinos diena; Tarptautinė darbo diena"),
            ("2022-06-05", "Tėvo diena"),
            ("2022-06-24", "Rasos ir Joninių diena"),
            (
                "2022-07-06",
                "Valstybės (Lietuvos karaliaus Mindaugo karūnavimo) ir Tautiškos giesmės diena",
            ),
            ("2022-08-15", "Žolinė (Švč. Mergelės Marijos ėmimo į dangų diena)"),
            ("2022-11-01", "Visų Šventųjų diena"),
            ("2022-11-02", "Mirusiųjų atminimo (Vėlinių) diena"),
            ("2022-12-24", "Kūčių diena"),
            ("2022-12-25", "Šv. Kalėdų pirma diena"),
            ("2022-12-26", "Šv. Kalėdų antra diena"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-02-16", "Day of Restoration of the State of Lithuania"),
            ("2022-03-11", "Day of Restoration of Independence of Lithuania"),
            ("2022-04-17", "Easter Sunday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-05-01", "International Workers' Day; Mother's Day"),
            ("2022-06-05", "Father's Day"),
            ("2022-06-24", "Day of Dew and Saint John"),
            ("2022-07-06", "Statehood Day"),
            ("2022-08-15", "Assumption Day"),
            ("2022-11-01", "All Saints' Day"),
            ("2022-11-02", "All Souls' Day"),
            ("2022-12-24", "Christmas Eve"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Second Day of Christmas"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-02-16", "День відновлення Литовської держави"),
            ("2022-03-11", "День відновлення незалежності Литви"),
            ("2022-04-17", "Великдень"),
            ("2022-04-18", "Великодній понеділок"),
            ("2022-05-01", "День матері; Міжнародний день трудящих"),
            ("2022-06-05", "День батька"),
            ("2022-06-24", "День роси та День Івана Купала"),
            ("2022-07-06", "День державності та День національного гімну"),
            ("2022-08-15", "Внебовзяття Пресвятої Діви Марії"),
            ("2022-11-01", "День усіх святих"),
            ("2022-11-02", "День памʼяті (День всіх померлих)"),
            ("2022-12-24", "Святий вечір"),
            ("2022-12-25", "Різдво Христове"),
            ("2022-12-26", "Другий день Різдва"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

"""Generate the JS code that sets the copyright text and updates the footer."""

from datetime import datetime, timezone

import mkdocs_gen_files


def main():
    now = datetime.now(tz=timezone.utc)
    copyright_text = f"© Copyright {now.year}. Last updated on {now:%b %d, %Y}."

    js_code = f"""// This file is generated at build time.
    document.addEventListener("DOMContentLoaded", function () {{
        const contentInfo = document.querySelector("footer div[role='contentinfo']");
        if (contentInfo) {{
            contentInfo.insertAdjacentHTML("beforebegin", "<p>{copyright_text}</p>");
        }}
    }});
    """

    with mkdocs_gen_files.open("js/copyright.js", "w", encoding="utf-8", newline="\n") as f:
        f.write(js_code)


if __name__ in {"__main__", "<run_path>"}:
    main()



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.guinea_bissau import GuineaBissau
from tests.common import CommonCountryTests


class TestGuineaBissau(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(GuineaBissau)

    def test_new_years_day(self):
        self.assertHolidayName("Ano Novo", (f"{year}-01-01" for year in self.full_range))

    def test_national_heroes_day(self):
        self.assertHolidayName(
            "Dia dos Heróis Nacionais", (f"{year}-01-20" for year in self.full_range)
        )

    def test_day_of_the_beginning_of_the_armed_struggle(self):
        self.assertHolidayName(
            "Dia do Início da Luta Armada", (f"{year}-01-23" for year in self.full_range)
        )

    def test_womens_day(self):
        self.assertHolidayName(
            "Dia Internacional da Mulher", (f"{year}-03-08" for year in self.full_range)
        )

    def test_easter_sunday(self):
        name = "Páscoa"
        self.assertHolidayName(
            name,
            "2023-04-09",
            "2024-03-31",
            "2025-04-20",
        )
        self.assertHolidayName(name, self.full_range)

    def test_workers_day(self):
        self.assertHolidayName("Dia do Trabalhador", (f"{year}-05-01" for year in self.full_range))

    def test_pidjiguiti_day(self):
        self.assertHolidayName("Dia de Pidjiguiti", (f"{year}-08-03" for year in self.full_range))

    def test_independence_day(self):
        self.assertHolidayName(
            "Dia da Independência", (f"{year}-09-24" for year in self.full_range)
        )

    def test_christmas_day(self):
        self.assertHolidayName("Dia de Natal", (f"{year}-12-25" for year in self.full_range))

    def test_eid_al_fitr(self):
        name = "Korité"
        self.assertHolidayName(
            name,
            "2023-04-21",
            "2024-04-10",
            "2025-03-30",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_eid_al_adha(self):
        name = "Tabaski"
        self.assertHolidayName(
            name,
            "2023-06-28",
            "2024-06-16",
            "2025-06-06",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_2024(self):
        self.assertHolidays(
            GuineaBissau(years=2024),
            ("2024-01-01", "Ano Novo"),
            ("2024-01-20", "Dia dos Heróis Nacionais"),
            ("2024-01-23", "Dia do Início da Luta Armada"),
            ("2024-03-08", "Dia Internacional da Mulher"),
            ("2024-03-31", "Páscoa"),
            ("2024-04-10", "Korité"),
            ("2024-05-01", "Dia do Trabalhador"),
            ("2024-06-16", "Tabaski"),
            ("2024-08-03", "Dia de Pidjiguiti"),
            ("2024-09-24", "Dia da Independência"),
            ("2024-12-25", "Dia de Natal"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2025-01-01", "Ano Novo"),
            ("2025-01-20", "Dia dos Heróis Nacionais"),
            ("2025-01-23", "Dia do Início da Luta Armada"),
            ("2025-03-08", "Dia Internacional da Mulher"),
            ("2025-03-30", "Korité"),
            ("2025-04-20", "Páscoa"),
            ("2025-05-01", "Dia do Trabalhador"),
            ("2025-06-06", "Tabaski"),
            ("2025-08-03", "Dia de Pidjiguiti"),
            ("2025-09-24", "Dia da Independência"),
            ("2025-12-25", "Dia de Natal"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2025-01-01", "New Year's Day"),
            ("2025-01-20", "National Heroes' Day"),
            ("2025-01-23", "Day of the Beginning of the Armed Struggle"),
            ("2025-03-08", "International Women's Day"),
            ("2025-03-30", "Eid al-Fitr"),
            ("2025-04-20", "Easter Sunday"),
            ("2025-05-01", "Worker's Day"),
            ("2025-06-06", "Eid al-Adha"),
            ("2025-08-03", "Pidjiguiti Day"),
            ("2025-09-24", "Independence Day"),
            ("2025-12-25", "Christmas Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

import unittest

from holidays.calendars.chinese import _ChineseLunisolar, KOREAN_CALENDAR, VIETNAMESE_CALENDAR


class TestChineseLunisolarCalendar(unittest.TestCase):
    def setUp(self) -> None:
        super().setUp()
        self.calendar = _ChineseLunisolar()

    def test_check_calendar(self):
        self.assertRaises(ValueError, lambda: _ChineseLunisolar("INVALID_CALENDAR"))

    def test_qingming_chinese(self):
        for year, day in (
            (1906, 6),
            (1910, 6),
            (1914, 5),
            (1939, 6),
            (1943, 6),
            (1947, 5),
            (1972, 5),
            (1976, 4),
            (1980, 4),
            (2005, 5),
            (2009, 4),
            (2013, 4),
            (2038, 5),
            (2042, 4),
            (2046, 4),
            (2071, 5),
            (2075, 4),
            (2079, 4),
        ):
            self.assertEqual(self.calendar.qingming_date(year)[0].day, day)

    def test_qingming_korean(self):
        for year, day in (
            (1910, 6),
            (1914, 6),
            (1918, 5),
            (1943, 6),
            (1947, 6),
            (1971, 5),
            (1980, 5),
            (1984, 4),
            (1988, 4),
            (2013, 5),
            (2017, 4),
            (2021, 4),
            (2042, 5),
            (2046, 4),
            (2050, 4),
            (2075, 5),
            (2079, 4),
            (2083, 4),
        ):
            self.assertEqual(self.calendar.qingming_date(year, KOREAN_CALENDAR)[0].day, day)

    def test_qingming_vietnamese(self):
        for year, day in (
            (1902, 6),
            (1906, 6),
            (1910, 5),
            (1931, 6),
            (1935, 6),
            (1939, 5),
            (1968, 5),
            (1972, 4),
            (1976, 4),
            (2001, 5),
            (2005, 4),
            (2009, 4),
            (2034, 5),
            (2038, 4),
            (2042, 4),
            (2063, 5),
            (2067, 4),
            (2071, 4),
        ):
            self.assertEqual(self.calendar.qingming_date(year, VIETNAMESE_CALENDAR)[0].day, day)

    def test_qingming_estimated(self):
        for year in (1900, 2100):
            self.assertTrue(self.calendar.qingming_date(year)[1])
        for year in (1901, 2099):
            self.assertFalse(self.calendar.qingming_date(year)[1])

    def test_winter_solstice_chinese(self):
        for year, day in (
            (1910, 23),
            (1914, 23),
            (1918, 22),
            (1943, 23),
            (1947, 23),
            (1951, 22),
            (1984, 22),
            (1988, 21),
            (1992, 21),
            (2017, 22),
            (2021, 21),
            (2025, 21),
            (2054, 22),
            (2058, 21),
            (2062, 21),
            (2087, 22),
            (2091, 21),
            (2095, 21),
        ):
            self.assertEqual(self.calendar.winter_solstice_date(year)[0].day, day)

    def test_winter_solstice_korean(self):
        for year, day in (
            (1914, 23),
            (1918, 23),
            (1922, 22),
            (1951, 23),
            (1955, 23),
            (1959, 22),
            (1988, 22),
            (1992, 21),
            (1996, 21),
            (2025, 22),
            (2029, 21),
            (2033, 21),
            (2058, 22),
            (2062, 21),
            (2066, 21),
            (2095, 22),
            (2099, 21),
        ):
            self.assertEqual(self.calendar.winter_solstice_date(year, KOREAN_CALENDAR)[0].day, day)

    def test_winter_solstice_vietnamese(self):
        for year, day in (
            (1906, 23),
            (1910, 23),
            (1914, 22),
            (1943, 23),
            (1947, 22),
            (1976, 22),
            (1980, 21),
            (1984, 21),
            (2013, 22),
            (2017, 21),
            (2021, 21),
            (2046, 22),
            (2050, 21),
            (2054, 21),
            (2079, 22),
            (2083, 21),
            (2087, 21),
        ):
            self.assertEqual(
                self.calendar.winter_solstice_date(year, VIETNAMESE_CALENDAR)[0].day, day
            )

    def test_winter_solstice_estimated(self):
        for year in (1900, 2100):
            self.assertTrue(self.calendar.winter_solstice_date(year)[1])
        for year in (1901, 2099):
            self.assertFalse(self.calendar.winter_solstice_date(year)[1])



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.croatia import Croatia
from tests.common import CommonCountryTests


class TestCroatia(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Croatia)

    def test_new_years_day(self):
        self.assertHolidayName("Nova godina", (f"{year}-01-01" for year in self.full_range))

    def test_epiphany(self):
        name = "Bogojavljenje ili Sveta tri kralja"
        self.assertHolidayName(
            name,
            (
                f"{year}-01-06"
                for year in (*range(self.start_year, 2002), *range(2003, self.end_year))
            ),
        )
        self.assertNoHolidayName(name, 2002)

    def test_easter(self):
        name = "Uskrs"
        self.assertHolidayName(
            name,
            "2009-04-12",
            "2019-04-21",
            "2020-04-12",
            "2021-04-04",
            "2022-04-17",
            "2023-04-09",
        )
        self.assertHolidayName(name, range(2009, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 2009))

    def test_easter_monday(self):
        name = "Uskrsni ponedjeljak"
        self.assertHolidayName(
            name,
            "2019-04-22",
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
        )
        self.assertHolidayName(name, self.full_range)

    def test_corpus_christi(self):
        name = "Tijelovo"
        self.assertHolidayName(
            name,
            "2002-05-30",
            "2019-06-20",
            "2020-06-11",
            "2021-06-03",
            "2022-06-16",
            "2023-06-08",
        )
        self.assertHolidayName(name, range(2002, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 2002))

    def test_labor_day(self):
        self.assertHolidayName("Praznik rada", (f"{year}-05-01" for year in self.full_range))

    def test_statehood_day(self):
        name = "Dan državnosti"
        self.assertHolidayName(name, (f"{year}-05-30" for year in range(1996, 2002)))
        self.assertHolidayName(name, (f"{year}-06-25" for year in range(2002, 2020)))
        self.assertHolidayName(name, (f"{year}-05-30" for year in range(2020, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1996))

    def test_antifascist_struggle_day(self):
        self.assertHolidayName(
            "Dan antifašističke borbe", (f"{year}-06-22" for year in self.full_range)
        )

    def test_victory_and_homeland_thanksgiving_day(self):
        name_1992 = "Dan pobjede i domovinske zahvalnosti"
        name_2008 = "Dan pobjede i domovinske zahvalnosti i Dan hrvatskih branitelja"
        self.assertHolidayName(
            name_1992, (f"{year}-08-05" for year in range(self.start_year, 2008))
        )
        self.assertHolidayName(name_2008, (f"{year}-08-05" for year in range(2008, self.end_year)))
        self.assertNoHolidayName(name_1992, range(2008, self.end_year))
        self.assertNoHolidayName(name_2008, range(self.start_year, 2008))

    def test_assumption_of_mary_day(self):
        self.assertHolidayName("Velika Gospa", (f"{year}-08-15" for year in self.full_range))

    def test_independence_day(self):
        name = "Dan neovisnosti"
        self.assertHolidayName(name, (f"{year}-10-08" for year in range(2002, 2020)))
        self.assertNoHolidayName(name, range(self.start_year, 2002), range(2020, self.end_year))

    def test_all_saints_day(self):
        self.assertHolidayName("Svi sveti", (f"{year}-11-01" for year in self.full_range))

    def test_remembrance_day(self):
        name = "Dan sjećanja na žrtve Domovinskog rata i Dan sjećanja na žrtvu Vukovara i Škabrnje"
        self.assertHolidayName(name, (f"{year}-11-18" for year in range(2020, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2020))

    def test_christmas_day(self):
        self.assertHolidayName("Božić", (f"{year}-12-25" for year in self.full_range))

    def test_saint_stephens_day(self):
        self.assertHolidayName("Sveti Stjepan", (f"{year}-12-26" for year in self.full_range))

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Nova godina"),
            ("2022-01-06", "Bogojavljenje ili Sveta tri kralja"),
            ("2022-04-17", "Uskrs"),
            ("2022-04-18", "Uskrsni ponedjeljak"),
            ("2022-05-01", "Praznik rada"),
            ("2022-05-30", "Dan državnosti"),
            ("2022-06-16", "Tijelovo"),
            ("2022-06-22", "Dan antifašističke borbe"),
            ("2022-08-05", "Dan pobjede i domovinske zahvalnosti i Dan hrvatskih branitelja"),
            ("2022-08-15", "Velika Gospa"),
            ("2022-11-01", "Svi sveti"),
            (
                "2022-11-18",
                "Dan sjećanja na žrtve Domovinskog rata i "
                "Dan sjećanja na žrtvu Vukovara i Škabrnje",
            ),
            ("2022-12-25", "Božić"),
            ("2022-12-26", "Sveti Stjepan"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-01-06", "Epiphany"),
            ("2022-04-17", "Easter Sunday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-05-01", "Labor Day"),
            ("2022-05-30", "Statehood Day"),
            ("2022-06-16", "Corpus Christi"),
            ("2022-06-22", "Anti-Fascist Struggle Day"),
            ("2022-08-05", "Victory and Homeland Thanksgiving Day and Croatian Veterans Day"),
            ("2022-08-15", "Assumption Day"),
            ("2022-11-01", "All Saints' Day"),
            ("2022-11-18", "Remembrance Day"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Saint Stephen's Day"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-01-06", "Богоявлення"),
            ("2022-04-17", "Великдень"),
            ("2022-04-18", "Великодній понеділок"),
            ("2022-05-01", "День праці"),
            ("2022-05-30", "День державності"),
            ("2022-06-16", "Свято Тіла і Крові Христових"),
            ("2022-06-22", "День антифашистської боротьби"),
            ("2022-08-05", "День перемоги і подяки вітчизні та День хорватських захисників"),
            ("2022-08-15", "Внебовзяття Пресвятої Діви Марії"),
            ("2022-11-01", "День усіх святих"),
            ("2022-11-18", "День памʼяті жертв Вітчизняної війни та жертв Вуковара і Шкабрні"),
            ("2022-12-25", "Різдво Христове"),
            ("2022-12-26", "День Святого Стефана"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.tuvalu import Tuvalu
from tests.common import CommonCountryTests


class TestTuvalu(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        years = range(1990, 2050)
        super().setUpClass(Tuvalu, years=years)
        cls.subdiv_holidays = {
            subdiv: Tuvalu(subdiv=subdiv, years=years) for subdiv in Tuvalu.subdivisions
        }

    def test_new_years(self):
        name = "Tausaga Fou"
        self.assertHolidayName(name, (f"{year}-01-01" for year in range(1990, 2050)))

    def test_commonwealth_day(self):
        name = "Aso Atefenua"
        dt = (
            "2001-03-12",
            "2002-03-11",
            "2003-03-10",
            "2004-03-08",
            "2005-03-14",
            "2006-03-13",
            "2007-03-12",
        )
        self.assertHolidayName(name, dt)
        self.assertHolidayName(name, range(1990, 2020))
        self.assertNoHolidayName(name, range(2021, 2050))

    def test_good_friday(self):
        name = "Aso toe tu"
        dt = (
            "1999-04-02",
            "2000-04-21",
            "2010-04-02",
            "2018-03-30",
            "2019-04-19",
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
        )
        self.assertHolidayName(name, dt)
        self.assertHolidayName(name, range(1990, 2050))

    def test_easter_monday(self):
        name = "Toe Tu aso gafua"
        dt = (
            "1999-04-05",
            "2000-04-24",
            "2010-04-05",
            "2018-04-02",
            "2019-04-22",
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
        )
        self.assertHolidayName(name, dt)
        self.assertHolidayName(name, range(1990, 2050))

    def test_gospel_day(self):
        name = "Te Aso o te Tala Lei"
        dt = (
            "2001-05-14",
            "2002-05-13",
            "2003-05-12",
            "2004-05-10",
            "2005-05-09",
            "2006-05-15",
            "2007-05-14",
        )
        self.assertHolidayName(name, dt)
        self.assertHolidayName(name, range(1990, 2050))

        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "NKL":
                self.assertHolidayName(
                    name, holidays, (f"{year}-05-10" for year in range(1990, 2050))
                )

    def test_king_birthday(self):
        name = "Asofanau Tupu"
        dt = (
            "2024-06-08",
            "2025-06-14",
            "2026-06-13",
            "2027-06-12",
            "2028-06-10",
            "2029-06-09",
            "2030-06-08",
        )
        self.assertHolidayName(name, dt)
        self.assertHolidayName(name, range(2023, 2050))
        self.assertNoHolidayName(name, range(1990, 2023))

    def test_queen_birthday(self):
        name = "Asofanau Fafine"
        dt = (
            "1990-06-09",
            "1991-06-08",
            "1992-06-13",
            "1999-06-12",
            "2002-06-08",
            "2007-06-09",
            "2008-06-14",
        )
        self.assertHolidayName(name, dt)
        self.assertHolidayName(name, range(1990, 2023))
        self.assertNoHolidayName(name, range(2023, 2050))

    def test_national_children_day(self):
        name = "Aso Tamaliki"
        dt = (
            "2012-08-06",
            "2013-08-05",
            "2014-08-04",
            "2015-08-03",
            "2016-08-01",
            "2020-10-12",
            "2021-10-11",
            "2022-10-10",
            "2023-10-09",
            "2024-10-14",
            "2025-10-13",
        )
        self.assertHolidayName(name, dt)
        self.assertHolidayName(name, range(1990, 2050))

    def test_national_youth_day(self):
        name = "Aso tupulaga"
        dt = (
            "2020-08-03",
            "2021-08-02",
            "2022-08-01",
            "2023-08-07",
            "2024-08-05",
            "2025-08-04",
        )
        self.assertHolidayName(name, dt)
        self.assertHolidayName(name, range(2019, 2050))
        self.assertNoHolidayName(name, range(1990, 2019))

    def test_tuvalu_day(self):
        name = "Tutokotasi"
        self.assertHolidayName(name, (f"{year}-10-01" for year in range(1990, 2050)))
        self.assertHolidayName(name, (f"{year}-10-02" for year in range(1990, 2050)))

    def test_heir_to_the_throne_birthday(self):
        name = "Aso fanau o te sui ote Tupu"
        dt = (
            "1990-11-12",
            "1996-11-11",
            "2000-11-13",
            "2001-11-12",
            "2009-11-09",
            "2015-11-09",
        )
        self.assertHolidayName(name, dt)
        self.assertHolidayName(name, range(1990, 2023))
        self.assertNoHolidayName(name, range(2023, 2050))

    def test_christmas_day(self):
        self.assertHolidayName("Kilisimasi", (f"{year}-12-25" for year in range(1990, 2050)))

    def test_boxing_day(self):
        self.assertHolidayName("Aso Faipele", (f"{year}-12-26" for year in range(1990, 2050)))

    def test_the_day_of_the_bombing(self):
        name = "Te Aso o te Paula"
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "FUN":
                self.assertHolidayName(
                    name, holidays, (f"{year}-04-23" for year in range(1990, 2050))
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_cyclone_day(self):
        name = "Aso o te matagi"
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "FUN":
                self.assertHolidayName(
                    name, holidays, (f"{year}-10-21" for year in range(1990, 2050))
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_niutao_day(self):
        name = "Te Aso o te Setema"
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "NIT":
                self.assertHolidayName(
                    name, holidays, (f"{year}-09-17" for year in range(1990, 2050))
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_nukufetau_day(self):
        name = "Te Aso O Tutasi"
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "NKF":
                self.assertHolidayName(
                    name, holidays, (f"{year}-02-11" for year in range(1990, 2050))
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_golden_jubilee(self):
        name = "Te Po o Tefolaha"
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "NMA":
                self.assertHolidayName(
                    name, holidays, (f"{year}-01-08" for year in range(1990, 2050))
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_big_day(self):
        name = "Po Lahi"
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "NMA":
                self.assertHolidayName(
                    name, holidays, (f"{year}-02-03" for year in range(1990, 2050))
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_nanumaga_day(self):
        name = "Aho o te Fakavae"
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "NMG":
                self.assertHolidayName(
                    name, holidays, (f"{year}-04-15" for year in range(1990, 2050))
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_day_of_the_flood(self):
        name = "Bogin te Ieka"
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "NUI":
                self.assertHolidayName(
                    name, holidays, (f"{year}-02-16" for year in range(1990, 2050))
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_happy_day(self):
        name = "Te Aso Fiafia"
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "VAI":
                self.assertHolidayName(
                    name, holidays, (f"{year}-11-25" for year in range(1990, 2050))
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_all_holidays(self):
        holidays_found = set()
        for subdiv in Tuvalu.subdivisions:
            holidays_found.update(
                Tuvalu(
                    subdiv=subdiv,
                    observed=False,
                    years=(1990, 2007, 2012, 2015, 2023),
                ).values()
            )
        all_holidays = {
            "Aho o te Fakavae",
            "Aso Atefenua",
            "Aso Faipele",
            "Aso fanau o te sui ote Tupu",
            "Aso o te matagi",
            "Aso Tamaliki",
            "Aso toe tu",
            "Aso tupulaga",
            "Asofanau Fafine",
            "Asofanau Tupu",
            "Bogin te Ieka",
            "Kilisimasi",
            "Po Lahi",
            "Tausaga Fou",
            "Te Aso Fiafia",
            "Te Aso o te Paula",
            "Te Aso o te Setema",
            "Te Aso o te Tala Lei",
            "Te Aso O Tutasi",
            "Te Po o Tefolaha",
            "Toe Tu aso gafua",
            "Tutokotasi",
        }
        self.assertEqual(all_holidays, holidays_found)

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Tausaga Fou"),
            ("2022-01-03", "Tausaga Fou (fakamatakuga)"),
            ("2022-01-08", "Te Po o Tefolaha"),
            ("2022-02-03", "Po Lahi"),
            ("2022-02-11", "Te Aso O Tutasi"),
            ("2022-02-16", "Bogin te Ieka"),
            ("2022-04-15", "Aho o te Fakavae; Aso toe tu"),
            ("2022-04-18", "Toe Tu aso gafua"),
            ("2022-04-23", "Te Aso o te Paula"),
            ("2022-05-09", "Te Aso o te Tala Lei"),
            ("2022-05-10", "Te Aso o te Tala Lei"),
            ("2022-06-11", "Asofanau Fafine"),
            ("2022-08-01", "Aso tupulaga"),
            ("2022-09-17", "Te Aso o te Setema"),
            ("2022-10-01", "Tutokotasi"),
            ("2022-10-02", "Tutokotasi"),
            ("2022-10-03", "Tutokotasi (fakamatakuga)"),
            ("2022-10-04", "Tutokotasi (fakamatakuga)"),
            ("2022-10-10", "Aso Tamaliki"),
            ("2022-10-21", "Aso o te matagi"),
            ("2022-11-14", "Aso fanau o te sui ote Tupu"),
            ("2022-11-25", "Te Aso Fiafia"),
            ("2022-12-25", "Kilisimasi"),
            ("2022-12-26", "Aso Faipele"),
            ("2022-12-27", "Kilisimasi (fakamatakuga)"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-01-03", "New Year's Day (observed)"),
            ("2022-01-08", "Golden Jubilee"),
            ("2022-02-03", "Big Day"),
            ("2022-02-11", "Nukufetau Day"),
            ("2022-02-16", "Day of the Flood"),
            ("2022-04-15", "Good Friday; Nanumaga Day"),
            ("2022-04-18", "Easter Monday"),
            ("2022-04-23", "The Day of the Bombing"),
            ("2022-05-09", "Gospel Day"),
            ("2022-05-10", "Gospel Day"),
            ("2022-06-11", "Queen's Birthday"),
            ("2022-08-01", "National Youth Day"),
            ("2022-09-17", "Niutao Day"),
            ("2022-10-01", "Tuvalu Day"),
            ("2022-10-02", "Tuvalu Day"),
            ("2022-10-03", "Tuvalu Day (observed)"),
            ("2022-10-04", "Tuvalu Day (observed)"),
            ("2022-10-10", "National Children's Day"),
            ("2022-10-21", "Cyclone Day"),
            ("2022-11-14", "Heir to the Throne's Birthday"),
            ("2022-11-25", "Happy Day"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Boxing Day"),
            ("2022-12-27", "Christmas Day (observed)"),
        )

    def test_l10n_en_gb(self):
        self.assertLocalizedHolidays(
            "en_GB",
            ("2022-01-01", "New Year's Day"),
            ("2022-01-03", "New Year's Day (observed)"),
            ("2022-01-08", "Golden Jubilee"),
            ("2022-02-03", "Big Day"),
            ("2022-02-11", "Nukufetau Day"),
            ("2022-02-16", "Day of the Flood"),
            ("2022-04-15", "Good Friday; Nanumaga Day"),
            ("2022-04-18", "Easter Monday"),
            ("2022-04-23", "The Day of the Bombing"),
            ("2022-05-09", "Gospel Day"),
            ("2022-05-10", "Gospel Day"),
            ("2022-06-11", "Queen's Birthday"),
            ("2022-08-01", "National Youth Day"),
            ("2022-09-17", "Niutao Day"),
            ("2022-10-01", "Tuvalu Day"),
            ("2022-10-02", "Tuvalu Day"),
            ("2022-10-03", "Tuvalu Day (observed)"),
            ("2022-10-04", "Tuvalu Day (observed)"),
            ("2022-10-10", "National Children's Day"),
            ("2022-10-21", "Cyclone Day"),
            ("2022-11-14", "Heir to the Throne's Birthday"),
            ("2022-11-25", "Happy Day"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Boxing Day"),
            ("2022-12-27", "Christmas Day (observed)"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.timor_leste import TimorLeste
from tests.common import CommonCountryTests


class TestTimorLeste(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(TimorLeste)

    def test_special_government_holidays(self):
        self.assertGovernmentHoliday(
            "2010-11-03",
            "2010-12-24",
            "2010-12-31",
            "2011-08-15",
            "2011-11-03",
            "2011-12-26",
            "2012-01-02",
            "2012-01-23",
            "2012-02-22",
            "2012-03-16",
            "2012-04-16",
            "2012-04-17",
            "2012-11-27",
            "2012-11-29",
            "2012-12-24",
            "2012-12-26",
            "2012-12-31",
            "2013-02-13",
            "2013-03-28",
            "2013-04-01",
            "2013-08-20",
            "2013-11-29",
            "2013-12-24",
            "2013-12-26",
            "2013-12-31",
            "2014-03-05",
            "2014-04-17",
            "2014-04-21",
            "2014-07-22",
            "2014-07-23",
            "2014-08-15",
            "2014-08-20",
            "2014-12-24",
            "2014-12-26",
            "2014-12-31",
            "2015-01-02",
            "2015-02-18",
            "2015-04-02",
            "2015-05-13",
            "2015-06-05",
            "2015-08-20",
            "2015-12-24",
            "2015-12-31",
            "2016-02-10",
            "2016-03-24",
            "2016-07-06",
            "2016-11-03",
            "2016-12-26",
            "2017-01-02",
            "2017-03-01",
            "2017-03-20",
            "2017-03-21",
            "2017-04-13",
            "2017-12-26",
            "2018-01-02",
            "2018-02-14",
            "2018-02-16",
            "2018-03-29",
            "2018-08-22",
            "2019-02-05",
            "2019-03-06",
            "2019-04-18",
            "2019-08-12",
            "2019-08-20",
            "2019-08-26",
            "2019-08-27",
            "2019-08-28",
            "2019-08-29",
            "2019-10-31",
            "2019-12-24",
            "2019-12-26",
            "2019-12-30",
            "2020-01-02",
            "2020-02-26",
            "2020-08-20",
            "2020-08-31",
            "2020-11-03",
            "2020-12-24",
            "2021-02-12",
            "2021-02-17",
            "2021-11-03",
            "2022-02-01",
            "2022-03-02",
            "2022-03-18",
            "2022-04-14",
            "2022-04-18",
            "2022-04-19",
            "2022-04-20",
            "2022-08-29",
            "2022-09-06",
            "2022-10-31",
            "2022-12-09",
            "2022-12-26",
            "2023-01-02",
            "2023-01-23",
            "2023-02-22",
            "2023-04-06",
            "2023-04-10",
            "2023-04-20",
            "2023-04-21",
            "2023-05-19",
            "2023-05-22",
            "2023-10-27",
            "2023-11-13",
            "2023-12-26",
            "2024-01-02",
            "2024-02-14",
            "2024-03-28",
            "2024-08-28",
            "2024-08-29",
            "2024-09-09",
            "2024-09-10",
            "2024-09-11",
            "2024-10-31",
            "2024-11-29",
            "2024-12-24",
            "2025-01-02",
            "2025-01-29",
        )

    def test_new_years_day(self):
        self.assertHolidayName("Dia de Ano Novo", (f"{year}-01-01" for year in self.full_range))

    def test_veterans_day(self):
        name = "Dia dos Veteranos"
        self.assertHolidayName(name, (f"{year}-03-03" for year in range(2017, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2017))

    def test_good_friday(self):
        name = "Sexta-Feira Santa"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_international_workers_day(self):
        self.assertHolidayName(
            "Dia Mundial do Trabalhador", (f"{year}-05-01" for year in self.full_range)
        )

    def test_restoration_of_independence_day(self):
        self.assertHolidayName(
            "Dia da Restauração da Independência", (f"{year}-05-20" for year in self.full_range)
        )

    def test_corpus_domini(self):
        name = "Festa do Corpo de Deus"
        self.assertHolidayName(
            name,
            "2020-06-11",
            "2021-06-03",
            "2022-06-16",
            "2023-06-08",
            "2024-05-30",
            "2025-06-19",
        )
        self.assertHolidayName(name, self.full_range)

    def test_popular_consultation_day(self):
        self.assertHolidayName(
            "Dia da Consulta Popular", (f"{year}-08-30" for year in self.full_range)
        )

    def test_all_saints_day(self):
        self.assertHolidayName(
            "Dia de Todos os Santos", (f"{year}-11-01" for year in self.full_range)
        )

    def test_all_souls_day(self):
        self.assertHolidayName(
            "Dia de Todos os Fiéis Defuntos", (f"{year}-11-02" for year in self.full_range)
        )

    def test_national_womens_day(self):
        name = "Dia Nacional da Mulher"
        self.assertHolidayName(name, (f"{year}-11-03" for year in range(2023, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2023))
        self.assertWorkdayHolidayName(
            name, (f"{year}-11-03" for year in range(self.start_year, 2023))
        )
        self.assertNoWorkdayHolidayName(name, range(2023, self.end_year))

    def test_national_youth_day(self):
        self.assertHolidayName(
            "Dia Nacional da Juventude", (f"{year}-11-12" for year in self.full_range)
        )

    def test_proclamation_of_independence_day(self):
        self.assertHolidayName(
            "Dia da Proclamação da Independência", (f"{year}-11-28" for year in self.full_range)
        )

    def test_memorial_day(self):
        name = "Dia da Memória"
        self.assertHolidayName(name, (f"{year}-12-07" for year in range(2017, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2017))

    def test_day_of_our_lady_of_immaculate_conception_and_timor_leste_patroness(self):
        self.assertHolidayName(
            "Dia de Nossa Senhora da Imaculada Conceição, padroeira de Timor-Leste",
            (f"{year}-12-08" for year in self.full_range),
        )

    def test_christmas_day(self):
        self.assertHolidayName("Dia de Natal", (f"{year}-12-25" for year in self.full_range))

    def test_national_heroes_day(self):
        self.assertHolidayName(
            "Dia dos Heróis Nacionais",
            (f"{year}-12-07" for year in range(self.start_year, 2017)),
            (f"{year}-12-31" for year in range(2017, self.end_year)),
        )

    def test_eid_al_fitr(self):
        name = "Idul Fitri"
        self.assertHolidayName(
            name,
            "2020-05-24",
            "2021-05-13",
            "2022-05-02",
            "2023-04-22",
            "2024-04-10",
            "2025-03-31",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_eid_al_adha(self):
        name = "Idul Adha"
        self.assertHolidayName(
            name,
            "2020-07-31",
            "2021-07-19",
            "2022-07-09",
            "2023-06-29",
            "2024-06-17",
            "2025-06-06",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_ash_wednesday(self):
        name = "Quarta-Feira de Cinzas"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(
            name,
            "2020-02-26",
            "2021-02-17",
            "2022-03-02",
            "2023-02-22",
            "2024-02-14",
            "2025-03-05",
        )
        self.assertWorkdayHolidayName(name, self.full_range)

    def test_maundy_thursday(self):
        name = "Quinta-Feira Santa"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(
            name,
            "2020-04-09",
            "2021-04-01",
            "2022-04-14",
            "2023-04-06",
            "2024-03-28",
            "2025-04-17",
        )
        self.assertWorkdayHolidayName(name, self.full_range)

    def test_the_day_of_ascension_of_jesus_christ_into_heaven(self):
        name = "Dia da Ascensão de Jesus Cristo ao Céu"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(
            name,
            "2020-05-21",
            "2021-05-13",
            "2022-05-26",
            "2023-05-18",
            "2024-05-09",
            "2025-05-29",
        )
        self.assertWorkdayHolidayName(name, self.full_range)

    def test_international_childrens_day(self):
        name = "Dia Mundial da Criança"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(name, (f"{year}-06-01" for year in self.full_range))

    def test_day_of_the_armed_forces_for_the_national_liberation_of_timor_leste_falintil(self):
        name = "Dia das Forças Armadas de Libertação Nacional de Timor-Leste (FALINTIL)"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(name, (f"{year}-08-20" for year in self.full_range))

    def test_international_human_rights_day(self):
        name = "Dia Mundial dos Direitos Humanos"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(name, (f"{year}-12-10" for year in self.full_range))

    def test_2011_public(self):
        # https://timor-leste.gov.tl/?p=4442&lang=en&lang=en
        self.assertHolidaysInYear(
            2011,
            ("2011-01-01", "Dia de Ano Novo"),
            ("2011-04-22", "Sexta-Feira Santa"),
            ("2011-05-01", "Dia Mundial do Trabalhador"),
            ("2011-05-20", "Dia da Restauração da Independência"),
            ("2011-06-23", "Festa do Corpo de Deus"),
            ("2011-08-30", "Dia da Consulta Popular"),
            ("2011-08-31", "Idul Fitri"),
            ("2011-11-01", "Dia de Todos os Santos"),
            ("2011-11-02", "Dia de Todos os Fiéis Defuntos"),
            ("2011-11-07", "Idul Adha"),
            ("2011-11-12", "Dia Nacional da Juventude"),
            ("2011-11-28", "Dia da Proclamação da Independência"),
            ("2011-12-07", "Dia dos Heróis Nacionais"),
            (
                "2011-12-08",
                "Dia de Nossa Senhora da Imaculada Conceição, padroeira de Timor-Leste",
            ),
            ("2011-12-25", "Dia de Natal"),
        )

    def test_2012_public(self):
        # https://timor-leste.gov.tl/?p=6269&lang=en
        self.assertHolidaysInYear(
            2012,
            ("2012-01-01", "Dia de Ano Novo"),
            ("2012-04-06", "Sexta-Feira Santa"),
            ("2012-05-01", "Dia Mundial do Trabalhador"),
            ("2012-05-20", "Dia da Restauração da Independência"),
            ("2012-06-07", "Festa do Corpo de Deus"),
            ("2012-08-20", "Idul Fitri"),
            ("2012-08-30", "Dia da Consulta Popular"),
            ("2012-10-26", "Idul Adha"),
            ("2012-11-01", "Dia de Todos os Santos"),
            ("2012-11-02", "Dia de Todos os Fiéis Defuntos"),
            ("2012-11-12", "Dia Nacional da Juventude"),
            ("2012-11-28", "Dia da Proclamação da Independência"),
            ("2012-12-07", "Dia dos Heróis Nacionais"),
            (
                "2012-12-08",
                "Dia de Nossa Senhora da Imaculada Conceição, padroeira de Timor-Leste",
            ),
            ("2012-12-25", "Dia de Natal"),
        )

    def test_2013_public(self):
        # https://timor-leste.gov.tl/?p=7627&lang=en&lang=en
        self.assertHolidaysInYear(
            2013,
            ("2013-01-01", "Dia de Ano Novo"),
            ("2013-03-29", "Sexta-Feira Santa"),
            ("2013-05-01", "Dia Mundial do Trabalhador"),
            ("2013-05-20", "Dia da Restauração da Independência"),
            ("2013-05-30", "Festa do Corpo de Deus"),
            ("2013-08-08", "Idul Fitri"),
            ("2013-08-30", "Dia da Consulta Popular"),
            ("2013-10-15", "Idul Adha"),
            ("2013-11-01", "Dia de Todos os Santos"),
            ("2013-11-02", "Dia de Todos os Fiéis Defuntos"),
            ("2013-11-12", "Dia Nacional da Juventude"),
            ("2013-11-28", "Dia da Proclamação da Independência"),
            ("2013-12-07", "Dia dos Heróis Nacionais"),
            (
                "2013-12-08",
                "Dia de Nossa Senhora da Imaculada Conceição, padroeira de Timor-Leste",
            ),
            ("2013-12-25", "Dia de Natal"),
        )

    def test_2014_public(self):
        # https://timor-leste.gov.tl/?p=9653&lang=en&lang=en
        self.assertHolidaysInYear(
            2014,
            ("2014-01-01", "Dia de Ano Novo"),
            ("2014-04-18", "Sexta-Feira Santa"),
            ("2014-05-01", "Dia Mundial do Trabalhador"),
            ("2014-05-20", "Dia da Restauração da Independência"),
            ("2014-06-19", "Festa do Corpo de Deus"),
            ("2014-07-28", "Idul Fitri"),
            ("2014-08-30", "Dia da Consulta Popular"),
            ("2014-10-04", "Idul Adha"),
            ("2014-11-01", "Dia de Todos os Santos"),
            ("2014-11-02", "Dia de Todos os Fiéis Defuntos"),
            ("2014-11-12", "Dia Nacional da Juventude"),
            ("2014-11-28", "Dia da Proclamação da Independência"),
            ("2014-12-07", "Dia dos Heróis Nacionais"),
            (
                "2014-12-08",
                "Dia de Nossa Senhora da Imaculada Conceição, padroeira de Timor-Leste",
            ),
            ("2014-12-25", "Dia de Natal"),
        )

    def test_2015_public(self):
        # https://timor-leste.gov.tl/?p=11331&lang=en&lang=en
        self.assertHolidaysInYear(
            2015,
            ("2015-01-01", "Dia de Ano Novo"),
            ("2015-04-03", "Sexta-Feira Santa"),
            ("2015-05-01", "Dia Mundial do Trabalhador"),
            ("2015-05-20", "Dia da Restauração da Independência"),
            ("2015-06-04", "Festa do Corpo de Deus"),
            ("2015-07-17", "Idul Fitri"),
            ("2015-08-30", "Dia da Consulta Popular"),
            ("2015-09-24", "Idul Adha"),
            ("2015-11-01", "Dia de Todos os Santos"),
            ("2015-11-02", "Dia de Todos os Fiéis Defuntos"),
            ("2015-11-12", "Dia Nacional da Juventude"),
            ("2015-11-28", "Dia da Proclamação da Independência"),
            ("2015-12-07", "Dia dos Heróis Nacionais"),
            (
                "2015-12-08",
                "Dia de Nossa Senhora da Imaculada Conceição, padroeira de Timor-Leste",
            ),
            ("2015-12-25", "Dia de Natal"),
        )

    def test_2016_public(self):
        # https://timor-leste.gov.tl/?p=14419&lang=en&lang=en
        self.assertHolidaysInYear(
            2016,
            ("2016-01-01", "Dia de Ano Novo"),
            ("2016-03-25", "Sexta-Feira Santa"),
            ("2016-05-01", "Dia Mundial do Trabalhador"),
            ("2016-05-20", "Dia da Restauração da Independência"),
            ("2016-05-26", "Festa do Corpo de Deus"),
            ("2016-07-07", "Idul Fitri"),
            ("2016-08-30", "Dia da Consulta Popular"),
            ("2016-09-18", "Idul Adha"),
            ("2016-11-01", "Dia de Todos os Santos"),
            ("2016-11-02", "Dia de Todos os Fiéis Defuntos"),
            ("2016-11-12", "Dia Nacional da Juventude"),
            ("2016-11-28", "Dia da Proclamação da Independência"),
            ("2016-12-07", "Dia dos Heróis Nacionais"),
            (
                "2016-12-08",
                "Dia de Nossa Senhora da Imaculada Conceição, padroeira de Timor-Leste",
            ),
            ("2016-12-25", "Dia de Natal"),
        )

    def test_2017_public(self):
        # https://timor-leste.gov.tl/?p=17138&lang=en&lang=en
        self.assertHolidaysInYear(
            2017,
            ("2017-01-01", "Dia de Ano Novo"),
            ("2017-03-03", "Dia dos Veteranos"),
            ("2017-04-14", "Sexta-Feira Santa"),
            ("2017-05-01", "Dia Mundial do Trabalhador"),
            ("2017-05-20", "Dia da Restauração da Independência"),
            ("2017-06-15", "Festa do Corpo de Deus"),
            ("2017-06-26", "Idul Fitri"),
            ("2017-08-30", "Dia da Consulta Popular"),
            ("2017-09-01", "Idul Adha"),
            ("2017-11-01", "Dia de Todos os Santos"),
            ("2017-11-02", "Dia de Todos os Fiéis Defuntos"),
            ("2017-11-12", "Dia Nacional da Juventude"),
            ("2017-11-28", "Dia da Proclamação da Independência"),
            ("2017-12-07", "Dia da Memória"),
            (
                "2017-12-08",
                "Dia de Nossa Senhora da Imaculada Conceição, padroeira de Timor-Leste",
            ),
            ("2017-12-25", "Dia de Natal"),
            ("2017-12-31", "Dia dos Heróis Nacionais"),
        )

    def test_2018_public(self):
        self.assertHolidaysInYear(
            2018,
            ("2018-01-01", "Dia de Ano Novo"),
            ("2018-03-03", "Dia dos Veteranos"),
            ("2018-03-30", "Sexta-Feira Santa"),
            ("2018-05-01", "Dia Mundial do Trabalhador"),
            ("2018-05-20", "Dia da Restauração da Independência"),
            ("2018-05-31", "Festa do Corpo de Deus"),
            ("2018-06-15", "Idul Fitri"),
            ("2018-08-21", "Idul Adha"),
            ("2018-08-30", "Dia da Consulta Popular"),
            ("2018-11-01", "Dia de Todos os Santos"),
            ("2018-11-02", "Dia de Todos os Fiéis Defuntos"),
            ("2018-11-12", "Dia Nacional da Juventude"),
            ("2018-11-28", "Dia da Proclamação da Independência"),
            ("2018-12-07", "Dia da Memória"),
            (
                "2018-12-08",
                "Dia de Nossa Senhora da Imaculada Conceição, padroeira de Timor-Leste",
            ),
            ("2018-12-25", "Dia de Natal"),
            ("2018-12-31", "Dia dos Heróis Nacionais"),
        )

    def test_2019_public(self):
        # https://timor-leste.gov.tl/?p=21146&lang=en&lang=en
        self.assertHolidaysInYear(
            2019,
            ("2019-01-01", "Dia de Ano Novo"),
            ("2019-03-03", "Dia dos Veteranos"),
            ("2019-04-19", "Sexta-Feira Santa"),
            ("2019-05-01", "Dia Mundial do Trabalhador"),
            ("2019-05-20", "Dia da Restauração da Independência"),
            ("2019-06-06", "Idul Fitri"),
            ("2019-06-20", "Festa do Corpo de Deus"),
            ("2019-08-11", "Idul Adha"),
            ("2019-08-30", "Dia da Consulta Popular"),
            ("2019-11-01", "Dia de Todos os Santos"),
            ("2019-11-02", "Dia de Todos os Fiéis Defuntos"),
            ("2019-11-12", "Dia Nacional da Juventude"),
            ("2019-11-28", "Dia da Proclamação da Independência"),
            ("2019-12-07", "Dia da Memória"),
            (
                "2019-12-08",
                "Dia de Nossa Senhora da Imaculada Conceição, padroeira de Timor-Leste",
            ),
            ("2019-12-25", "Dia de Natal"),
            ("2019-12-31", "Dia dos Heróis Nacionais"),
        )

    def test_2020_public(self):
        # https://timor-leste.gov.tl/?p=23415&lang=en&lang=en
        self.assertHolidaysInYear(
            2020,
            ("2020-01-01", "Dia de Ano Novo"),
            ("2020-03-03", "Dia dos Veteranos"),
            ("2020-04-10", "Sexta-Feira Santa"),
            ("2020-05-01", "Dia Mundial do Trabalhador"),
            ("2020-05-20", "Dia da Restauração da Independência"),
            ("2020-05-24", "Idul Fitri"),
            ("2020-06-11", "Festa do Corpo de Deus"),
            ("2020-07-31", "Idul Adha"),
            ("2020-08-30", "Dia da Consulta Popular"),
            ("2020-11-01", "Dia de Todos os Santos"),
            ("2020-11-02", "Dia de Todos os Fiéis Defuntos"),
            ("2020-11-12", "Dia Nacional da Juventude"),
            ("2020-11-28", "Dia da Proclamação da Independência"),
            ("2020-12-07", "Dia da Memória"),
            (
                "2020-12-08",
                "Dia de Nossa Senhora da Imaculada Conceição, padroeira de Timor-Leste",
            ),
            ("2020-12-25", "Dia de Natal"),
            ("2020-12-31", "Dia dos Heróis Nacionais"),
        )

    def test_2021_public(self):
        # https://timor-leste.gov.tl/?p=26494&lang=en&lang=en
        self.assertHolidaysInYear(
            2021,
            ("2021-01-01", "Dia de Ano Novo"),
            ("2021-03-03", "Dia dos Veteranos"),
            ("2021-04-02", "Sexta-Feira Santa"),
            ("2021-05-01", "Dia Mundial do Trabalhador"),
            ("2021-05-13", "Idul Fitri"),
            ("2021-05-20", "Dia da Restauração da Independência"),
            ("2021-06-03", "Festa do Corpo de Deus"),
            ("2021-07-19", "Idul Adha"),
            ("2021-08-30", "Dia da Consulta Popular"),
            ("2021-11-01", "Dia de Todos os Santos"),
            ("2021-11-02", "Dia de Todos os Fiéis Defuntos"),
            ("2021-11-12", "Dia Nacional da Juventude"),
            ("2021-11-28", "Dia da Proclamação da Independência"),
            ("2021-12-07", "Dia da Memória"),
            (
                "2021-12-08",
                "Dia de Nossa Senhora da Imaculada Conceição, padroeira de Timor-Leste",
            ),
            ("2021-12-25", "Dia de Natal"),
            ("2021-12-31", "Dia dos Heróis Nacionais"),
        )

    def test_2022_public(self):
        # https://timor-leste.gov.tl/?p=30266&lang=en
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "Dia de Ano Novo"),
            ("2022-03-03", "Dia dos Veteranos"),
            ("2022-04-15", "Sexta-Feira Santa"),
            ("2022-05-01", "Dia Mundial do Trabalhador"),
            ("2022-05-02", "Idul Fitri"),
            ("2022-05-20", "Dia da Restauração da Independência"),
            ("2022-06-16", "Festa do Corpo de Deus"),
            ("2022-07-09", "Idul Adha"),
            ("2022-08-30", "Dia da Consulta Popular"),
            ("2022-11-01", "Dia de Todos os Santos"),
            ("2022-11-02", "Dia de Todos os Fiéis Defuntos"),
            ("2022-11-12", "Dia Nacional da Juventude"),
            ("2022-11-28", "Dia da Proclamação da Independência"),
            ("2022-12-07", "Dia da Memória"),
            (
                "2022-12-08",
                "Dia de Nossa Senhora da Imaculada Conceição, padroeira de Timor-Leste",
            ),
            ("2022-12-25", "Dia de Natal"),
            ("2022-12-31", "Dia dos Heróis Nacionais"),
        )

    def test_2023_public(self):
        # https://timor-leste.gov.tl/?p=31750&lang=en&lang=en
        self.assertHolidaysInYear(
            2023,
            ("2023-01-01", "Dia de Ano Novo"),
            ("2023-03-03", "Dia dos Veteranos"),
            ("2023-04-07", "Sexta-Feira Santa"),
            ("2023-04-22", "Idul Fitri"),
            ("2023-05-01", "Dia Mundial do Trabalhador"),
            ("2023-05-20", "Dia da Restauração da Independência"),
            ("2023-06-08", "Festa do Corpo de Deus"),
            ("2023-06-29", "Idul Adha"),
            ("2023-08-30", "Dia da Consulta Popular"),
            ("2023-11-01", "Dia de Todos os Santos"),
            ("2023-11-02", "Dia de Todos os Fiéis Defuntos"),
            ("2023-11-03", "Dia Nacional da Mulher"),
            ("2023-11-12", "Dia Nacional da Juventude"),
            ("2023-11-28", "Dia da Proclamação da Independência"),
            ("2023-12-07", "Dia da Memória"),
            (
                "2023-12-08",
                "Dia de Nossa Senhora da Imaculada Conceição, padroeira de Timor-Leste",
            ),
            ("2023-12-25", "Dia de Natal"),
            ("2023-12-31", "Dia dos Heróis Nacionais"),
        )

    def test_2024_public(self):
        # https://timor-leste.gov.tl/?p=35833&lang=en&lang=en
        self.assertHolidaysInYear(
            2024,
            ("2024-01-01", "Dia de Ano Novo"),
            ("2024-03-03", "Dia dos Veteranos"),
            ("2024-03-29", "Sexta-Feira Santa"),
            ("2024-04-10", "Idul Fitri"),
            ("2024-05-01", "Dia Mundial do Trabalhador"),
            ("2024-05-20", "Dia da Restauração da Independência"),
            ("2024-05-30", "Festa do Corpo de Deus"),
            ("2024-06-17", "Idul Adha"),
            ("2024-08-30", "Dia da Consulta Popular"),
            ("2024-11-01", "Dia de Todos os Santos"),
            ("2024-11-02", "Dia de Todos os Fiéis Defuntos"),
            ("2024-11-03", "Dia Nacional da Mulher"),
            ("2024-11-12", "Dia Nacional da Juventude"),
            ("2024-11-28", "Dia da Proclamação da Independência"),
            ("2024-12-07", "Dia da Memória"),
            (
                "2024-12-08",
                "Dia de Nossa Senhora da Imaculada Conceição, padroeira de Timor-Leste",
            ),
            ("2024-12-25", "Dia de Natal"),
            ("2024-12-31", "Dia dos Heróis Nacionais"),
        )

    def test_2025_public(self):
        # https://timor-leste.gov.tl/?p=41492&lang=en
        self.assertHolidaysInYear(
            2025,
            ("2025-01-01", "Dia de Ano Novo"),
            ("2025-03-03", "Dia dos Veteranos"),
            ("2025-03-31", "Idul Fitri"),
            ("2025-04-18", "Sexta-Feira Santa"),
            ("2025-05-01", "Dia Mundial do Trabalhador"),
            ("2025-05-20", "Dia da Restauração da Independência"),
            ("2025-06-06", "Idul Adha"),
            ("2025-06-19", "Festa do Corpo de Deus"),
            ("2025-08-30", "Dia da Consulta Popular"),
            ("2025-11-01", "Dia de Todos os Santos"),
            ("2025-11-02", "Dia de Todos os Fiéis Defuntos"),
            ("2025-11-03", "Dia Nacional da Mulher"),
            ("2025-11-12", "Dia Nacional da Juventude"),
            ("2025-11-28", "Dia da Proclamação da Independência"),
            ("2025-12-07", "Dia da Memória"),
            (
                "2025-12-08",
                "Dia de Nossa Senhora da Imaculada Conceição, padroeira de Timor-Leste",
            ),
            ("2025-12-25", "Dia de Natal"),
            ("2025-12-31", "Dia dos Heróis Nacionais"),
        )

    def test_2022_workday(self):
        self.assertWorkdayHolidaysInYear(
            2022,
            ("2022-03-02", "Quarta-Feira de Cinzas"),
            ("2022-04-14", "Quinta-Feira Santa"),
            ("2022-05-26", "Dia da Ascensão de Jesus Cristo ao Céu"),
            ("2022-06-01", "Dia Mundial da Criança"),
            (
                "2022-08-20",
                "Dia das Forças Armadas de Libertação Nacional de Timor-Leste (FALINTIL)",
            ),
            ("2022-11-03", "Dia Nacional da Mulher"),
            ("2022-12-10", "Dia Mundial dos Direitos Humanos"),
        )

    def test_2023_workday(self):
        self.assertWorkdayHolidaysInYear(
            2023,
            ("2023-02-22", "Quarta-Feira de Cinzas"),
            ("2023-04-06", "Quinta-Feira Santa"),
            ("2023-05-18", "Dia da Ascensão de Jesus Cristo ao Céu"),
            ("2023-06-01", "Dia Mundial da Criança"),
            (
                "2023-08-20",
                "Dia das Forças Armadas de Libertação Nacional de Timor-Leste (FALINTIL)",
            ),
            ("2023-12-10", "Dia Mundial dos Direitos Humanos"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2023-01-01", "Dia de Ano Novo"),
            ("2023-01-02", "Feriados Nacionais (Especiais)"),
            ("2023-01-23", "Feriados Nacionais (Especiais)"),
            ("2023-02-22", "Feriados Nacionais (Especiais); Quarta-Feira de Cinzas"),
            ("2023-03-03", "Dia dos Veteranos"),
            ("2023-04-06", "Feriados Nacionais (Especiais); Quinta-Feira Santa"),
            ("2023-04-07", "Sexta-Feira Santa"),
            ("2023-04-10", "Feriados Nacionais (Especiais)"),
            ("2023-04-20", "Feriados Nacionais (Especiais)"),
            ("2023-04-21", "Feriados Nacionais (Especiais)"),
            ("2023-04-22", "Idul Fitri"),
            ("2023-05-01", "Dia Mundial do Trabalhador"),
            ("2023-05-18", "Dia da Ascensão de Jesus Cristo ao Céu"),
            ("2023-05-19", "Dia de Eleições Parlamentares"),
            ("2023-05-20", "Dia da Restauração da Independência"),
            ("2023-05-22", "Dia de Eleições Parlamentares"),
            ("2023-06-01", "Dia Mundial da Criança"),
            ("2023-06-08", "Festa do Corpo de Deus"),
            ("2023-06-29", "Idul Adha"),
            (
                "2023-08-20",
                "Dia das Forças Armadas de Libertação Nacional de Timor-Leste (FALINTIL)",
            ),
            ("2023-08-30", "Dia da Consulta Popular"),
            ("2023-10-27", "Dia de eleições locais"),
            ("2023-11-01", "Dia de Todos os Santos"),
            ("2023-11-02", "Dia de Todos os Fiéis Defuntos"),
            ("2023-11-03", "Dia Nacional da Mulher"),
            ("2023-11-12", "Dia Nacional da Juventude"),
            ("2023-11-13", "Dia de eleições locais"),
            ("2023-11-28", "Dia da Proclamação da Independência"),
            ("2023-12-07", "Dia da Memória"),
            (
                "2023-12-08",
                "Dia de Nossa Senhora da Imaculada Conceição, padroeira de Timor-Leste",
            ),
            ("2023-12-10", "Dia Mundial dos Direitos Humanos"),
            ("2023-12-25", "Dia de Natal"),
            ("2023-12-26", "Feriados Nacionais (Especiais)"),
            ("2023-12-31", "Dia dos Heróis Nacionais"),
        )

    def test_l10n_en_tl(self):
        self.assertLocalizedHolidays(
            "en_TL",
            ("2023-01-01", "New Year's Day"),
            ("2023-01-02", "National Holidays (Special)"),
            ("2023-01-23", "National Holidays (Special)"),
            ("2023-02-22", "Ash Wednesday; National Holidays (Special)"),
            ("2023-03-03", "Veterans Day"),
            ("2023-04-06", "Holy Thursday; National Holidays (Special)"),
            ("2023-04-07", "Holy Friday"),
            ("2023-04-10", "National Holidays (Special)"),
            ("2023-04-20", "National Holidays (Special)"),
            ("2023-04-21", "National Holidays (Special)"),
            ("2023-04-22", "Idul Fitri"),
            ("2023-05-01", "World Labour Day"),
            ("2023-05-18", "The Day of Ascension of Jesus Christ into Heaven"),
            ("2023-05-19", "Parliamentary Election Day"),
            ("2023-05-20", "Restoration of Independence Day"),
            ("2023-05-22", "Parliamentary Election Day"),
            ("2023-06-01", "World Children's Day"),
            ("2023-06-08", "Corpus Christi"),
            ("2023-06-29", "Idul Adha"),
            (
                "2023-08-20",
                "Day of the Armed Forces for the National Liberation of Timor-Leste (FALINTIL)",
            ),
            ("2023-08-30", "Popular Consultation Day"),
            ("2023-10-27", "Local Election Day"),
            ("2023-11-01", "All Saints Day"),
            ("2023-11-02", "All Souls Day"),
            ("2023-11-03", "National Women's Day"),
            ("2023-11-12", "National Youth Day"),
            ("2023-11-13", "Local Election Day"),
            ("2023-11-28", "Proclamation of Independence Day"),
            ("2023-12-07", "Memorial Day"),
            ("2023-12-08", "Day of Our Lady of Immaculate Conception and Timor-Leste Patroness"),
            ("2023-12-10", "World Human Rights Day"),
            ("2023-12-25", "Christmas Day"),
            ("2023-12-26", "National Holidays (Special)"),
            ("2023-12-31", "National Heroes Day"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2023-01-01", "New Year's Day"),
            ("2023-01-02", "National Holidays (Special)"),
            ("2023-01-23", "National Holidays (Special)"),
            ("2023-02-22", "Ash Wednesday; National Holidays (Special)"),
            ("2023-03-03", "Veteran's Day"),
            ("2023-04-06", "Maundy Thursday; National Holidays (Special)"),
            ("2023-04-07", "Good Friday"),
            ("2023-04-10", "National Holidays (Special)"),
            ("2023-04-20", "National Holidays (Special)"),
            ("2023-04-21", "National Holidays (Special)"),
            ("2023-04-22", "Eid al-Fitr"),
            ("2023-05-01", "International Worker's Day"),
            ("2023-05-18", "The Day of Ascension of Jesus Christ into Heaven"),
            ("2023-05-19", "Parliamentary Election Day"),
            ("2023-05-20", "Restoration of Independence Day"),
            ("2023-05-22", "Parliamentary Election Day"),
            ("2023-06-01", "International Children's Day"),
            ("2023-06-08", "Corpus Christi"),
            ("2023-06-29", "Eid al-Adha"),
            (
                "2023-08-20",
                "Day of the Armed Forces for the National Liberation of Timor-Leste (FALINTIL)",
            ),
            ("2023-08-30", "Popular Consultation Day"),
            ("2023-10-27", "Local Election Day"),
            ("2023-11-01", "All Saints' Day"),
            ("2023-11-02", "All Souls' Day"),
            ("2023-11-03", "National Women's Day"),
            ("2023-11-12", "National Youth Day"),
            ("2023-11-13", "Local Election Day"),
            ("2023-11-28", "Proclamation of Independence Day"),
            ("2023-12-07", "Memorial Day"),
            ("2023-12-08", "Day of Our Lady of Immaculate Conception and Timor-Leste Patroness"),
            ("2023-12-10", "International Human Rights Day"),
            ("2023-12-25", "Christmas Day"),
            ("2023-12-26", "National Holidays (Special)"),
            ("2023-12-31", "National Heroes Day"),
        )

    def test_l10n_tet(self):
        self.assertLocalizedHolidays(
            "tet",
            ("2023-01-01", "Loron Tinan-Foun nian"),
            ("2023-01-02", "Feriadu Nasional (Espesial)"),
            ("2023-01-23", "Feriadu Nasional (Espesial)"),
            ("2023-02-22", "Feriadu Nasional (Espesial); Kuarta-Feira Sinzas"),
            ("2023-03-03", "Loron Veteranu sira nian"),
            ("2023-04-06", "Feriadu Nasional (Espesial); Quinta-Feira Santa"),
            ("2023-04-07", "Sesta-Feira Santa"),
            ("2023-04-10", "Feriadu Nasional (Espesial)"),
            ("2023-04-20", "Feriadu Nasional (Espesial)"),
            ("2023-04-21", "Feriadu Nasional (Espesial)"),
            ("2023-04-22", "Idul-Fitri"),
            ("2023-05-01", "Loron Mundiál Serbisu-na'in sira nian"),
            ("2023-05-18", "Loron Ascensão do Senhor Jesus Cristo hi'it An ba Lalehan nian"),
            ("2023-05-19", "Loron Eleisaun Parlamentár nian"),
            ("2023-05-20", "Loron Restaurasaun Independénsia nian"),
            ("2023-05-22", "Loron Eleisaun Parlamentár nian"),
            ("2023-06-01", "Loron Mundial ba Labarik"),
            ("2023-06-08", "Festa Korpu de Deus"),
            ("2023-06-29", "Idul Adha"),
            ("2023-08-20", "Loron Forsa Armada Libertasaun Nasionál Timor-Leste (FALINTIL) nian"),
            ("2023-08-30", "Loron Konsulta Populár nian"),
            ("2023-10-27", "Loron eleisaun lokál nian"),
            ("2023-11-01", "Loron Santu sira Hotu nian"),
            ("2023-11-02", "Loron Matebian sira nian"),
            ("2023-11-03", "Loron Nasionál Feto"),
            ("2023-11-12", "Loron Nasionál Foin-Sa'e sira nian"),
            ("2023-11-13", "Loron eleisaun lokál nian"),
            ("2023-11-28", "Loron Proklamasaun Independénsia nian"),
            ("2023-12-07", "Loron Memória nian"),
            ("2023-12-08", "Loron Nossa Senhora da Imaculada Conceição, mahein Timor-Leste nian"),
            ("2023-12-10", "Loron Mundiál Direitu Umanu"),
            ("2023-12-25", "Loron Natál"),
            ("2023-12-26", "Feriadu Nasional (Espesial)"),
            ("2023-12-31", "Loron Eroi Nasionál sira nian"),
        )

    def test_l10n_th(self):
        self.assertLocalizedHolidays(
            "th",
            ("2023-01-01", "วันขึ้นปีใหม่"),
            ("2023-01-02", "วันหยุดพิเศษ (เพิ่มเติม)"),
            ("2023-01-23", "วันหยุดพิเศษ (เพิ่มเติม)"),
            ("2023-02-22", "วันพุธรับเถ้า; วันหยุดพิเศษ (เพิ่มเติม)"),
            ("2023-03-03", "วันทหารผ่านศึก"),
            ("2023-04-06", "วันพฤหัสศักดิ์สิทธิ์; วันหยุดพิเศษ (เพิ่มเติม)"),
            ("2023-04-07", "วันศุกร์ประเสริฐ"),
            ("2023-04-10", "วันหยุดพิเศษ (เพิ่มเติม)"),
            ("2023-04-20", "วันหยุดพิเศษ (เพิ่มเติม)"),
            ("2023-04-21", "วันหยุดพิเศษ (เพิ่มเติม)"),
            ("2023-04-22", "วันอีฎิ้ลฟิตริ"),
            ("2023-05-01", "วันแรงงานสากล"),
            ("2023-05-18", "วันสมโภชพระเยซูเจ้าเสด็จขึ้นสวรรค์"),
            ("2023-05-19", "วันเลือกตั้งสมาชิกรัฐสภา"),
            ("2023-05-20", "วันรำลึกการกอบกู้เอกราชติมอร์-เลสเต"),
            ("2023-05-22", "วันเลือกตั้งสมาชิกรัฐสภา"),
            ("2023-06-01", "วันเด็กสากล"),
            ("2023-06-08", "วันสมโภชพระคริสตวรกาย"),
            ("2023-06-29", "วันอีดิ้ลอัฎฮา"),
            ("2023-08-20", "วันกองกำลังปลดปล่อยแห่งชาติติมอร์-เลสเต (FALINTIL)"),
            ("2023-08-30", "วันรำลึกการลงประชามติเอกราช"),
            ("2023-10-27", "วันเลือกตั้งท้องถิ่น"),
            ("2023-11-01", "วันสมโภชนักบุญทั้งหลาย"),
            ("2023-11-02", "วันภาวนาอุทิศแด่ผู้ล่วงลับ"),
            ("2023-11-03", "วันสตรีแห่งชาติ"),
            ("2023-11-12", "วันเยาวชนแห่งชาติ"),
            ("2023-11-13", "วันเลือกตั้งท้องถิ่น"),
            ("2023-11-28", "วันประกาศเอกราชติมอร์-เลสเต"),
            ("2023-12-07", "วันรำลึกวีรชน"),
            ("2023-12-08", "วันสมโภชแม่พระผู้ปฏิสนธินิรมลและแม่พระองค์อุปถัมภ์แห่งติมอร์-เลสเต"),
            ("2023-12-10", "วันสิทธิมนุษยชนสากล"),
            ("2023-12-25", "วันคริสต์มาส"),
            ("2023-12-26", "วันหยุดพิเศษ (เพิ่มเติม)"),
            ("2023-12-31", "วันวีรบุรุษแห่งชาติ"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.financial.european_central_bank import EuropeanCentralBank
from tests.common import CommonFinancialTests


class TestEuropeanCentralBank(CommonFinancialTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(EuropeanCentralBank)

    def test_special_holidays(self):
        self.assertHoliday(
            "1999-12-31",
            "2001-12-31",
        )

    def test_new_years_day(self):
        self.assertHolidayName("New Year's Day", (f"{year}-01-01" for year in self.full_range))

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2000-04-21",
            "2010-04-02",
            "2018-03-30",
            "2019-04-19",
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
        )
        self.assertHolidayName(name, range(2000, self.end_year))
        self.assertNoHolidayName(name, 1999)

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2000-04-24",
            "2010-04-05",
            "2018-04-02",
            "2019-04-22",
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
        )
        self.assertHolidayName(name, range(2000, self.end_year))
        self.assertNoHolidayName(name, 1999)

    def test_labor_day(self):
        name = "Labour Day"
        self.assertHolidayName(name, (f"{year}-05-01" for year in range(2000, self.end_year)))
        self.assertNoHolidayName(name, 1999)

    def test_christmas_day(self):
        self.assertHolidayName("Christmas Day", (f"{year}-12-25" for year in self.full_range))

    def test_christmas_holiday(self):
        name = "Christmas Holiday"
        self.assertHolidayName(name, (f"{year}-12-26" for year in range(2000, self.end_year)))
        self.assertNoHolidayName(name, 1999)

    def test_1999(self):
        self.assertHolidays(
            EuropeanCentralBank(years=1999),
            ("1999-01-01", "New Year's Day"),
            ("1999-12-25", "Christmas Day"),
            ("1999-12-31", "Additional closing day"),
        )

    def test_2015(self):
        self.assertHolidays(
            EuropeanCentralBank(years=2015),
            ("2015-01-01", "New Year's Day"),
            ("2015-04-03", "Good Friday"),
            ("2015-04-06", "Easter Monday"),
            ("2015-05-01", "Labour Day"),
            ("2015-12-25", "Christmas Day"),
            ("2015-12-26", "Christmas Holiday"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.constants import PUBLIC, WORKDAY
from holidays.groups import ChristianHolidays, InternationalHolidays, IslamicHolidays
from holidays.holiday_base import HolidayBase


class Benin(HolidayBase, ChristianHolidays, InternationalHolidays, IslamicHolidays):
    """Benin holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Benin>
        * [Order 71-14](https://web.archive.org/web/20240727075244/https://sgg.gouv.bj/doc/ordonnance-1971-14/)
        * [Order 76-35](https://web.archive.org/web/20240726042853/https://sgg.gouv.bj/doc/ordonnance-1976-35/)
        * [Order 79-33](https://web.archive.org/web/20240725115717/https://sgg.gouv.bj/doc/ordonnance-1979-33/)
        * [Law 90-019](https://web.archive.org/web/20250527021602/https://sgg.gouv.bj/doc/loi-90-019/)
        * [Law 97-031](https://web.archive.org/web/20250527022039/https://sgg.gouv.bj/doc/loi-97-031/)
        * [Law 2024-32](https://web.archive.org/web/20250321041430/https://sgg.gouv.bj/doc/loi-2024-32/)

    Official dates:
        * [2015](https://web.archive.org/web/20211130121041/https://beninconsulate.co.ke/index.php/news-events/7-public-holidays-2015)
        * [2017](https://web.archive.org/web/20211130124252/https://beninconsulate.co.ke/index.php/news-events/8-public-holidays-2017)
        * [2019](https://web.archive.org/web/20231003144018/https://beninconsulate.co.ke/index.php/news-events)
        * [2024](https://web.archive.org/web/20241211032923/https://beninconsulate.co.ke/index.php/news-events)
        * [2025](https://web.archive.org/web/20250317141957/https://beninconsulate.co.ke/index.php/news-events?start=1)

    Certain holidays dates:
        * 2022:
            * [All Saints' Day](https://web.archive.org/web/20241010101230/https://travail.gouv.bj/page/communiques/la-journee-du-mardi-1er-novembre-2022-declaree-feriee-chomee-et-payee)
            * [Christmas Day](https://web.archive.org/web/20240814200022/https://travail.gouv.bj/page/communiques/la-journee-du-dimanche-25-decembre-2022-declaree-feriee-chomee-et-payee-a-loccasion-de-la-celebration-de-la-fete-de-noel-lire-le-communique-du-ministre-du-travail-et-de-la-fonction-publique)
        * 2023:
            * [New Year's Day](https://web.archive.org/web/20240724084416/https://travail.gouv.bj/page/communiques/la-journee-du-dimanche-1er-janvier-2023-declaree-feriee-chomee-et-payee-a-loccasion-de-la-celebration-de-la-fete-du-nouvel-an)
            * [Vodoun Festival](https://web.archive.org/web/20240724084406/https://travail.gouv.bj/page/communiques/la-journee-du-mardi-10-janvier-2023-est-declaree-feriee-chomee-et-payee-a-loccasion-de-la-celebration-de-la-fete-annuelle-des-religions-traditionnelles-2301071350-956)
            * [Eid al-Fitr](https://web.archive.org/web/20240723004550/https://travail.gouv.bj/page/communiques/celebration-le-vendredi-21-avril-2023-de-la-fete-du-ramadan)
            * [Easter Monday](https://web.archive.org/web/20241010103541/https://travail.gouv.bj/page/communiques/la-journee-du-lundi-10-avril-2023-est-declaree-feriee-chomee-et-payee-en-raison-de-la-fete-de-paques)
            * [Whit Monday](https://web.archive.org/web/20240911163554/https://travail.gouv.bj/page/communiques/la-fete-de-pentecote)
            * [Labor Day](https://web.archive.org/web/20240814205702/https://travail.gouv.bj/page/communiques/la-journee-du-lundi-1er-mai-2023-declaree-feriee-chomee-et-payee-sur-toute-letendue-du-territoire-national-a-loccasion-de-la-celebration-de-la-fete-du-travail)
            * [Assumption Day](https://web.archive.org/web/20241214182235/https://travail.gouv.bj/page/communiques/la-journee-du-mardi-15-aout-2023-declaree-feriee-chomee-et-payee-en-raison-de-la-fete-de-lassomption)
            * [Independence Day](https://web.archive.org/web/20240723004533/https://travail.gouv.bj/page/communiques/la-journee-du-mardi-1er-aout-2023-declaree-feriee-chomee-et-payee-a-l-occasion-de-la-fete-nationale)
            * [Eid al-Adha](https://web.archive.org/web/20250422035008/https://travail.gouv.bj/page/communiques/communique-du-ministere-du-travail-et-de-la-fonction-publique-2306262022-744)
        * 2024:
            * [Vodoun Festival](https://web.archive.org/web/20240723004435/https://travail.gouv.bj/page/communiques/la-journee-du-mercredi-10-janvier-2024-est-feriee-chomee-et-payee-sur-toute-letendue-du-territoire-national)
            * [Eid al-Fitr](https://web.archive.org/web/20241010114534/https://travail.gouv.bj/page/communiques/la-journee-du-mercredi-10-avril-2024-jour-du-ramadan-est-declaree-feriee-chomee-et-payee-sur-toute-letendue-du-territoire-national)
            * [Easter Monday](https://web.archive.org/web/20241106004344/https://travail.gouv.bj/page/communiques/la-journee-du-lundi-1er-avril-2024-lundi-de-paques-est-chomee)
            * [Assumption Day](https://web.archive.org/web/20250215044724/https://travail.gouv.bj/page/communiques/la-journee-du-dimanche-15-aout-jour-de-lassomption-est-feriee-chomee-et-payee-sur-toute-letendue-du-territoire-national)
            * [Eid al-Adha](https://web.archive.org/web/20241214200230/https://travail.gouv.bj/page/communiques/jour-de-la-tabaski)
            * [Labor Day](https://web.archive.org/web/20250418224409/https://www.travail.gouv.bj/page/communiques/la-journee-du-mercredi-1er-mai-2024-journee-de-la-fete-du-travail-est-feriee-chomee-et-payee-sur-toute-letendue-du-territoire-national)
            * [Ascension Day](https://web.archive.org/web/20241214204011/https://travail.gouv.bj/page/communiques/le-jeudi-09-mai-2024-jour-de-lascension-est-feriee-chomee-et-payee-sur-toute-letendue-du-territoire-national)
            * [Whit Monday](https://web.archive.org/web/20241010113401/https://travail.gouv.bj/page/communiques/le-lundi-20-mai-2024-lundi-de-pentecote-est-ferie-chome-et-paye-sur-toute-letendue-du-territoire-national)
            * [Prophet's Birthday](https://web.archive.org/web/20250215034616/https://travail.gouv.bj/page/communiques/la-journee-du-dimanche-15-septembre-2024-jour-de-maoloud-est-feriee-chomee-et-payee-sur-toute-letendue-du-territoire-national)
            * [Christmas Day](https://web.archive.org/web/20250122081816/https://travail.gouv.bj/page/communiques/la-journee-du-mercredi-25-decembre-2024-est-feriee-chomee-et-payee-sur-toute-letendue-du-territoire-national)
        * 2025:
            * [New Year's Day](https://web.archive.org/web/20250422023225/https://travail.gouv.bj/page/communiques/la-journee-du-mercredi-1er-janvier-2025-fete-du-nouvel-an-est-feriee-chomee-et-payee-sur-toute-letendue-du-territoire-national)
            * [Vodoun Festival](https://web.archive.org/web/20250422034718/https://travail.gouv.bj/page/communiques/les-journees-du-jeudi-9-janvier-2024-et-vendredi-10-janvier-2024-sont-feriees-chomees-et-payees-sur-toute-letendue-du-territoire-national)
    """

    country = "BJ"
    default_language = "fr_BJ"
    # %s (estimated).
    estimated_label = tr("%s (estimé)")
    # Order 76-35 of June 30, 1976.
    start_year = 1977
    supported_categories = (PUBLIC, WORKDAY)
    supported_languages = ("en_US", "fr_BJ")

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.
        """
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        IslamicHolidays.__init__(self, show_estimated=islamic_show_estimated)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("Fête du Nouvel An"))

        # Established in 1998 by Law 97-031.
        # Changed in 2025 by Law 2024-32.
        if self._year >= 1998:
            # Vodoun Festival.
            name = tr("Fête annuelle des religions traditionnelles")
            if self._year >= 2025:
                self._add_holiday_1_day_prior_2nd_fri_of_jan(name)
                self._add_holiday_2nd_fri_of_jan(name)
            else:
                self._add_holiday_jan_10(name)

        # Established in 1980 by Order 79-33.
        # Abolished in 1991 by Law 90-019.
        if 1980 <= self._year <= 1990:
            # Martyrs' Day.
            self._add_holiday_jan_16(tr("Journée des Martyrs"))

            # Youth Day.
            self._add_holiday_apr_1(tr("Journée de la Jeunesse Béninoise"))

        # Changed in 1991 by Law 90-019.
        if self._year >= 1991:
            # Easter Monday.
            self._add_easter_monday(tr("Lundi de Pâques"))
        else:
            # Easter Sunday.
            self._add_easter_sunday(tr("Jour de Pâques"))

        # Labor Day.
        self._add_labor_day(tr("Fête du Travail"))

        # Established in 1991 by Law 90-019.
        if self._year >= 1991:
            # Ascension Day.
            self._add_ascension_thursday(tr("Jour de l'Ascension"))

        if self._year >= 1991:
            # Whit Monday.
            self._add_whit_monday(tr("Lundi de Pentecôte"))
        else:
            # Whit Sunday.
            self._add_whit_sunday(tr("Jour de Pentecôte"))

        # Established in 1990 by Law 90-019.
        if self._year >= 1990:
            # Assumption Day.
            self._add_assumption_of_mary_day(tr("Jour de l'Assomption"))

        # Established in 1977 by Order 71-14.
        # Abolished in 1990 by Law 90-019.
        if self._year <= 1989:
            # Armed Forces Day.
            self._add_holiday_oct_26(tr("Fête des Forces Armées Populaires du Bénin"))

        # Established in 1990 by Law 90-019.
        if self._year >= 1990:
            # All Saints' Day.
            self._add_all_saints_day(tr("La Toussaint"))

        # National Day.
        name = tr("Fête Nationale")
        if self._year >= 1991:
            self._add_holiday_aug_1(name)
        else:
            self._add_holiday_nov_30(name)

        # Christmas Day.
        self._add_christmas_day(tr("Jour de Noël"))

        # Established in 1977 by Order 71-14.
        # Abolished in 1990 by Law 90-019.
        if self._year <= 1989:
            # Production Day.
            self._add_holiday_dec_31(tr("Fête de la Production"))

        # Islamic holidays.

        # Established in 1990 by Law 90-019.
        if self._year >= 1990:
            # Prophet's Birthday.
            self._add_mawlid_day(tr("Journée Maouloud"))

        # Eid al-Fitr.
        self._add_eid_al_fitr_day(tr("Jour du Ramadan"))

        # Eid al-Adha.
        self._add_eid_al_adha_day(tr("Jour de la Tabaski"))

    def _populate_workday_holidays(self):
        if self._year >= 1990:
            # Remembrance Day.
            self._add_holiday_jan_16(tr("Journée de Souvenir"))

            # People's Sovereignty Day.
            self._add_holiday_feb_28(tr("Journée de la Souveraineté de Peuple"))

            # Women's Day.
            self._add_holiday_mar_8(tr("Journée de la Femme"))


class BJ(Benin):
    pass


class BEN(Benin):
    pass
